@@ -6,7 +6,7 @@
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-#      http://www.apache.org/licenses/LICENSE-2.0
+#      https://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
@@ -46,7 +46,7 @@ int LLVMFuzzerTestOneInput(const uint8_t * data, size_t size)
 	assert(res >= 0);
 
 	/*res = gnutls_set_default_priority(session);*/
-	res = gnutls_priority_set_direct(session, "NORMAL:"VERS_STR, NULL);
+	res = gnutls_priority_set_direct(session, "NORMAL:+SIGN-EDDSA-ED448:"VERS_STR, NULL);
 	assert(res >= 0);
 
 	memdata.data = data;
