@@ -26,8 +26,8 @@
 
 #include "fuzzer.h"
 
-static int cb(void *ctx, unsigned tls_id, const unsigned char *data,
-	      unsigned data_size)
+static
+int cb(void *ctx, unsigned tls_id, const unsigned char *data, unsigned data_size)
 {
 	return 0;
 }
@@ -41,11 +41,9 @@ int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size)
 
 	gnutls_ext_raw_parse(NULL, cb, &raw, 0);
 
-	gnutls_ext_raw_parse(NULL, cb, &raw,
-			     GNUTLS_EXT_RAW_FLAG_TLS_CLIENT_HELLO);
+	gnutls_ext_raw_parse(NULL, cb, &raw, GNUTLS_EXT_RAW_FLAG_TLS_CLIENT_HELLO);
 
-	gnutls_ext_raw_parse(NULL, cb, &raw,
-			     GNUTLS_EXT_RAW_FLAG_DTLS_CLIENT_HELLO);
+	gnutls_ext_raw_parse(NULL, cb, &raw, GNUTLS_EXT_RAW_FLAG_DTLS_CLIENT_HELLO);
 
 	return 0;
 }
