@@ -1,5 +1,5 @@
-#ifndef AES_ARM_H
-#define AES_ARM_H
+#ifndef GNUTLS_LIB_ACCELERATED_AARCH64_AES_AARCH64_H
+#define GNUTLS_LIB_ACCELERATED_AARCH64_AES_AARCH64_H
 
 #include "gnutls_int.h"
 
@@ -20,6 +20,21 @@ typedef struct {
 	if (s != 16 && s != 24 && s != 32) \
 		return GNUTLS_E_INVALID_REQUEST
 
+#include <intprops.h>
+#define AES_GCM_ENCRYPT_MAX_BYTES ((1ULL << 36) - 32)
+static inline int
+record_aes_gcm_encrypt_size(size_t *counter, size_t size) {
+	size_t sum;
+
+	if (!INT_ADD_OK(*counter, size, &sum) ||
+	    sum > AES_GCM_ENCRYPT_MAX_BYTES) {
+		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+	}
+	*counter = sum;
+
+	return 0;
+}
+
 int aes_v8_set_encrypt_key(const unsigned char *userKey, int bits, AES_KEY *key);  
 int aes_v8_set_decrypt_key(const unsigned char *userKey, int bits, AES_KEY *key);
 void aes_v8_cbc_encrypt(const unsigned char *in, unsigned char *out,
@@ -31,4 +46,4 @@ extern const gnutls_crypto_cipher_st _gnutls_aes_gcm_aarch64;
 extern const gnutls_crypto_cipher_st _gnutls_aes_cbc_aarch64;
 extern const gnutls_crypto_cipher_st _gnutls_aes_ccm_aarch64;
 
-#endif
+#endif /* GNUTLS_LIB_ACCELERATED_AARCH64_AES_AARCH64_H */
