@@ -28,17 +28,17 @@
 #include <nettle/sha.h>
 #include <nettle/macros.h>
 #include <nettle/nettle-meta.h>
-#include "sha-aarch64.h"
-#include "aarch64-common.h"
+#include <sha-aarch64.h>
+#include <aarch64-common.h>
 
 void sha1_block_data_order(void *c, const void *p, size_t len);
 void sha256_block_data_order(void *c, const void *p, size_t len);
 void sha512_block_data_order(void *c, const void *p, size_t len);
 
-typedef void (*update_func)(void *, size_t, const uint8_t *);
-typedef void (*digest_func)(void *, size_t, uint8_t *);
-typedef void (*set_key_func)(void *, size_t, const uint8_t *);
-typedef void (*init_func)(void *);
+typedef void (*update_func) (void *, size_t, const uint8_t *);
+typedef void (*digest_func) (void *, size_t, uint8_t *);
+typedef void (*set_key_func) (void *, size_t, const uint8_t *);
+typedef void (*init_func) (void *);
 
 struct aarch64_hash_ctx {
 	union {
@@ -56,8 +56,8 @@ struct aarch64_hash_ctx {
 	init_func init;
 };
 
-static int wrap_aarch64_hash_update(void *_ctx, const void *text,
-				    size_t textsize)
+static int
+wrap_aarch64_hash_update(void *_ctx, const void *text, size_t textsize)
 {
 	struct aarch64_hash_ctx *ctx = _ctx;
 
@@ -72,7 +72,7 @@ static void wrap_aarch64_hash_deinit(void *hd)
 }
 
 void aarch64_sha1_update(struct sha1_ctx *ctx, size_t length,
-			 const uint8_t *data)
+		     const uint8_t * data)
 {
 	struct {
 		uint32_t h0, h1, h2, h3, h4;
@@ -105,11 +105,12 @@ void aarch64_sha1_update(struct sha1_ctx *ctx, size_t length,
 	length -= res;
 
 	if (length > 0) {
+
 		t2 = length / SHA1_DATA_SIZE;
 
 		sha1_block_data_order(&octx, data, t2);
 
-		for (i = 0; i < t2; i++)
+		for (i=0;i<t2;i++)
 			ctx->count++;
 		data += length;
 	}
@@ -126,10 +127,11 @@ void aarch64_sha1_update(struct sha1_ctx *ctx, size_t length,
 	if (res > 0) {
 		sha1_update(ctx, res, data);
 	}
+
 }
 
 void aarch64_sha256_update(struct sha256_ctx *ctx, size_t length,
-			   const uint8_t *data)
+		     const uint8_t * data)
 {
 	struct {
 		uint32_t h[8];
@@ -160,8 +162,8 @@ void aarch64_sha256_update(struct sha256_ctx *ctx, size_t length,
 	if (length > 0) {
 		t2 = length / SHA1_DATA_SIZE;
 		sha256_block_data_order(&octx, data, t2);
-
-		for (i = 0; i < t2; i++)
+		
+		for (i=0;i<t2;i++)
 			ctx->count++;
 		data += length;
 	}
@@ -177,14 +179,14 @@ void aarch64_sha256_update(struct sha256_ctx *ctx, size_t length,
 }
 
 void aarch64_sha512_update(struct sha512_ctx *ctx, size_t length,
-			   const uint8_t *data)
+		     const uint8_t * data)
 {
 	struct {
 		uint64_t h[8];
 		uint64_t Nl, Nh;
 		union {
 			uint64_t d[16];
-			uint8_t p[16 * 8];
+			uint8_t p[16*8];
 		} u;
 		unsigned int num;
 		unsigned md_len;
@@ -211,8 +213,8 @@ void aarch64_sha512_update(struct sha512_ctx *ctx, size_t length,
 	if (length > 0) {
 		t2 = length / SHA512_DATA_SIZE;
 		sha512_block_data_order(&octx, data, t2);
-
-		for (i = 0; i < t2; i++)
+		
+		for (i=0;i<t2;i++)
 			MD_INCR(ctx);
 		data += length;
 	}
@@ -233,41 +235,41 @@ static int _ctx_init(gnutls_digest_algorithm_t algo,
 	switch (algo) {
 	case GNUTLS_DIG_SHA1:
 		sha1_init(&ctx->ctx.sha1);
-		ctx->update = (update_func)aarch64_sha1_update;
-		ctx->digest = (digest_func)sha1_digest;
-		ctx->init = (init_func)sha1_init;
+		ctx->update = (update_func) aarch64_sha1_update;
+		ctx->digest = (digest_func) sha1_digest;
+		ctx->init = (init_func) sha1_init;
 		ctx->ctx_ptr = &ctx->ctx.sha1;
 		ctx->length = SHA1_DIGEST_SIZE;
 		break;
 	case GNUTLS_DIG_SHA224:
 		sha224_init(&ctx->ctx.sha224);
-		ctx->update = (update_func)aarch64_sha256_update;
-		ctx->digest = (digest_func)sha224_digest;
-		ctx->init = (init_func)sha224_init;
+		ctx->update = (update_func) aarch64_sha256_update;
+		ctx->digest = (digest_func) sha224_digest;
+		ctx->init = (init_func) sha224_init;
 		ctx->ctx_ptr = &ctx->ctx.sha224;
 		ctx->length = SHA224_DIGEST_SIZE;
 		break;
 	case GNUTLS_DIG_SHA256:
 		sha256_init(&ctx->ctx.sha256);
-		ctx->update = (update_func)aarch64_sha256_update;
-		ctx->digest = (digest_func)sha256_digest;
-		ctx->init = (init_func)sha256_init;
+		ctx->update = (update_func) aarch64_sha256_update;
+		ctx->digest = (digest_func) sha256_digest;
+		ctx->init = (init_func) sha256_init;
 		ctx->ctx_ptr = &ctx->ctx.sha256;
 		ctx->length = SHA256_DIGEST_SIZE;
 		break;
 	case GNUTLS_DIG_SHA384:
 		sha384_init(&ctx->ctx.sha384);
-		ctx->update = (update_func)aarch64_sha512_update;
-		ctx->digest = (digest_func)sha384_digest;
-		ctx->init = (init_func)sha384_init;
+		ctx->update = (update_func) aarch64_sha512_update;
+		ctx->digest = (digest_func) sha384_digest;
+		ctx->init = (init_func) sha384_init;
 		ctx->ctx_ptr = &ctx->ctx.sha384;
 		ctx->length = SHA384_DIGEST_SIZE;
 		break;
 	case GNUTLS_DIG_SHA512:
 		sha512_init(&ctx->ctx.sha512);
-		ctx->update = (update_func)aarch64_sha512_update;
-		ctx->digest = (digest_func)sha512_digest;
-		ctx->init = (init_func)sha512_init;
+		ctx->update = (update_func) aarch64_sha512_update;
+		ctx->digest = (digest_func) sha512_digest;
+		ctx->init = (init_func) sha512_init;
 		ctx->ctx_ptr = &ctx->ctx.sha512;
 		ctx->length = SHA512_DIGEST_SIZE;
 		break;
@@ -279,6 +281,7 @@ static int _ctx_init(gnutls_digest_algorithm_t algo,
 	return 0;
 }
 
+
 static int wrap_aarch64_hash_init(gnutls_digest_algorithm_t algo, void **_ctx)
 {
 	struct aarch64_hash_ctx *ctx;
@@ -293,7 +296,6 @@ static int wrap_aarch64_hash_init(gnutls_digest_algorithm_t algo, void **_ctx)
 	ctx->algo = algo;
 
 	if ((ret = _ctx_init(algo, ctx)) < 0) {
-		gnutls_free(ctx);
 		gnutls_assert();
 		return ret;
 	}
@@ -303,10 +305,11 @@ static int wrap_aarch64_hash_init(gnutls_digest_algorithm_t algo, void **_ctx)
 	return 0;
 }
 
-static void *wrap_aarch64_hash_copy(const void *_ctx)
+static void *
+wrap_aarch64_hash_copy(const void *_ctx)
 {
 	struct aarch64_hash_ctx *new_ctx;
-	const struct aarch64_hash_ctx *ctx = _ctx;
+	const struct aarch64_hash_ctx *ctx=_ctx;
 	ptrdiff_t off = (uint8_t *)ctx->ctx_ptr - (uint8_t *)(&ctx->ctx);
 
 	new_ctx = gnutls_malloc(sizeof(struct aarch64_hash_ctx));
@@ -321,17 +324,12 @@ static void *wrap_aarch64_hash_copy(const void *_ctx)
 	return new_ctx;
 }
 
-static int wrap_aarch64_hash_output(void *src_ctx, void *digest,
-				    size_t digestsize)
+static int
+wrap_aarch64_hash_output(void *src_ctx, void *digest, size_t digestsize)
 {
 	struct aarch64_hash_ctx *ctx;
 	ctx = src_ctx;
 
-	if (digest == NULL) {
-		ctx->init(ctx->ctx_ptr);
-		return 0;
-	}
-
 	if (digestsize < ctx->length)
 		return gnutls_assert_val(GNUTLS_E_SHORT_MEMORY_BUFFER);
 
@@ -341,8 +339,8 @@ static int wrap_aarch64_hash_output(void *src_ctx, void *digest,
 }
 
 static int wrap_aarch64_hash_fast(gnutls_digest_algorithm_t algo,
-				  const void *text, size_t text_size,
-				  void *digest)
+				 const void *text, size_t text_size,
+				 void *digest)
 {
 	struct aarch64_hash_ctx ctx;
 	int ret;
@@ -358,16 +356,16 @@ static int wrap_aarch64_hash_fast(gnutls_digest_algorithm_t algo,
 }
 
 const struct nettle_hash aarch64_sha1 =
-	NN_HASH(sha1, aarch64_sha1_update, sha1_digest, SHA1);
+NN_HASH(sha1, aarch64_sha1_update, sha1_digest, SHA1);
 const struct nettle_hash aarch64_sha224 =
-	NN_HASH(sha224, aarch64_sha256_update, sha224_digest, SHA224);
+NN_HASH(sha224, aarch64_sha256_update, sha224_digest, SHA224);
 const struct nettle_hash aarch64_sha256 =
-	NN_HASH(sha256, aarch64_sha256_update, sha256_digest, SHA256);
+NN_HASH(sha256, aarch64_sha256_update, sha256_digest, SHA256);
 
 const struct nettle_hash aarch64_sha384 =
-	NN_HASH(sha384, aarch64_sha512_update, sha384_digest, SHA384);
+NN_HASH(sha384, aarch64_sha512_update, sha384_digest, SHA384);
 const struct nettle_hash aarch64_sha512 =
-	NN_HASH(sha512, aarch64_sha512_update, sha512_digest, SHA512);
+NN_HASH(sha512, aarch64_sha512_update, sha512_digest, SHA512);
 
 const gnutls_crypto_digest_st _gnutls_sha_aarch64 = {
 	.init = wrap_aarch64_hash_init,
