@@ -1,5 +1,5 @@
-#ifndef AES_PADLOCK_H
-#define AES_PADLOCK_H
+#ifndef GNUTLS_LIB_ACCELERATED_X86_AES_PADLOCK_H
+#define GNUTLS_LIB_ACCELERATED_X86_AES_PADLOCK_H
 
 #include "gnutls_int.h"
 #include <aes-x86.h>
@@ -30,8 +30,8 @@ struct padlock_ctx {
 extern const gnutls_crypto_cipher_st _gnutls_aes_padlock;
 extern const gnutls_crypto_cipher_st _gnutls_aes_gcm_padlock;
 
-extern const gnutls_crypto_mac_st _gnutls_hmac_sha_padlock;
-extern const gnutls_crypto_digest_st _gnutls_sha_padlock;
+extern const gnutls_crypto_mac_st _gnutls_hmac_sha_padlock_oneshot;
+extern const gnutls_crypto_digest_st _gnutls_sha_padlock_oneshot;
 
 int padlock_aes_cipher_setkey(void *_ctx, const void *userkey,
 			      size_t keysize);
@@ -43,4 +43,4 @@ int padlock_ecb_encrypt(void *out, const void *inp,
 			struct padlock_cipher_data *ctx, size_t len);
 int padlock_cbc_encrypt(void *out, const void *inp,
 			struct padlock_cipher_data *ctx, size_t len);
-#endif
+#endif /* GNUTLS_LIB_ACCELERATED_X86_AES_PADLOCK_H */
