@@ -20,16 +20,23 @@
  *
  */
 
-#ifndef AUTH_PSK_H
-#define AUTH_PSK_H
+#ifndef GNUTLS_LIB_AUTH_PSK_H
+#define GNUTLS_LIB_AUTH_PSK_H
 
 #include <auth.h>
 #include <auth/dh_common.h>
 
+#define _gnutls_copy_psk_username(info, datum) \
+	_gnutls_copy_psk_string(&(info)->username, &(info)->username_len, (datum))
+
+#define _gnutls_copy_psk_hint(info, datum) \
+	_gnutls_copy_psk_string(&(info)->hint, &(info)->hint_len, (datum))
+
 typedef struct gnutls_psk_client_credentials_st {
 	gnutls_datum_t username;
 	gnutls_datum_t key;
-	gnutls_psk_client_credentials_function *get_function;
+	gnutls_psk_client_credentials_function2 *get_function;
+	gnutls_psk_client_credentials_function *get_function_legacy;
 	/* TLS 1.3 - The HMAC algorithm to use to compute the binder values */
 	const mac_entry_st *binder_algo;
 } psk_client_credentials_st;
@@ -39,7 +46,8 @@ typedef struct gnutls_psk_server_credentials_st {
 	/* callback function, instead of reading the
 	 * password files.
 	 */
-	gnutls_psk_server_credentials_function *pwd_callback;
+	gnutls_psk_server_credentials_function2 *pwd_callback;
+	gnutls_psk_server_credentials_function *pwd_callback_legacy;
 
 	/* For DHE_PSK */
 	gnutls_dh_params_t dh_params;
@@ -56,15 +64,36 @@ typedef struct gnutls_psk_server_credentials_st {
 	const mac_entry_st *binder_algo;
 } psk_server_cred_st;
 
-/* these structures should not use allocated data */
 typedef struct psk_auth_info_st {
-	char username[MAX_USERNAME_SIZE + 1];
+	char *username;
+	uint16_t username_len;
 	dh_info_st dh;
-	char hint[MAX_USERNAME_SIZE + 1];
+	char *hint;
+	uint16_t hint_len;
 } *psk_auth_info_t;
 
 typedef struct psk_auth_info_st psk_auth_info_st;
 
+inline static int
+_gnutls_copy_psk_string(char **dest, uint16_t *dest_len, const gnutls_datum_t str)
+{
+	char *_tmp;
+
+	assert(MAX_USERNAME_SIZE >= str.size);
+	
+	_tmp = gnutls_malloc(str.size + 1);
+	if (_tmp == NULL)
+		return GNUTLS_E_MEMORY_ERROR;
+	memcpy(_tmp, str.data, str.size);
+	_tmp[str.size] = '\0';
+	
+	gnutls_free(*dest);
+	*dest = _tmp;
+	*dest_len = str.size;
+	
+	return GNUTLS_E_SUCCESS;
+}
+
 #ifdef ENABLE_PSK
 
 int
@@ -74,9 +103,8 @@ int _gnutls_gen_psk_server_kx(gnutls_session_t session,
 			      gnutls_buffer_st * data);
 int _gnutls_gen_psk_client_kx(gnutls_session_t, gnutls_buffer_st *);
 
-
 #else
 #define _gnutls_set_psk_session_key(x,y,z) GNUTLS_E_UNIMPLEMENTED_FEATURE
 #endif				/* ENABLE_PSK */
 
-#endif
+#endif /* GNUTLS_LIB_AUTH_PSK_H */
