@@ -16,13 +16,13 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>
  *
  */
 
-#include <gnutls_int.h>
-#include <gnutls_errors.h>
-#include <gnutls_datum.h>
+#include "gnutls_int.h"
+#include "errors.h"
+#include <datum.h>
 #include <auth/srp_passwd.h>
 
 #ifdef ENABLE_SRP
@@ -58,7 +58,7 @@ static const uint8_t asciitable[128] = {
 	0xff, 0xff
 };
 
-inline static int encode(uint8_t * result, const uint8_t * rdata, int left)
+inline static int encode(uint8_t * result, const uint8_t * rdata, unsigned left)
 {
 
 	int data_len;
@@ -127,7 +127,7 @@ inline static int encode(uint8_t * result, const uint8_t * rdata, int left)
 		}
 		break;
 	default:
-		return -1;
+		return GNUTLS_E_BASE64_ENCODING_ERROR;
 	}
 
 	return ret;
@@ -143,7 +143,7 @@ _gnutls_sbase64_encode(uint8_t * data, size_t data_size, char **result)
 	unsigned i, j;
 	int ret, tmp;
 	uint8_t tmpres[4];
-	int mod = data_size % 3;
+	unsigned mod = data_size % 3;
 
 	ret = mod;
 	if (ret != 0)
@@ -155,7 +155,7 @@ _gnutls_sbase64_encode(uint8_t * data, size_t data_size, char **result)
 
 	(*result) = gnutls_calloc(1, ret + 1);
 	if ((*result) == NULL)
-		return -1;
+		return GNUTLS_E_MEMORY_ERROR;
 
 	i = j = 0;
 /* encode the bytes that are not a multiple of 3 
@@ -203,7 +203,7 @@ inline static int decode(uint8_t * result, const uint8_t * data)
 	if (a1 != 0xff)
 		result[2] = a1 & 0xff;
 	else
-		return -1;
+		return GNUTLS_E_BASE64_DECODING_ERROR;
 	if (a2 != 0xff)
 		result[2] |= ((a2 & 0x03) << 6) & 0xff;
 
@@ -252,7 +252,7 @@ _gnutls_sbase64_decode(char *data, size_t idata_size, uint8_t ** result)
 
 	(*result) = gnutls_malloc(ret + 1);
 	if ((*result) == NULL)
-		return -1;
+		return GNUTLS_E_MEMORY_ERROR;
 
 	/* the first "block" is treated with special care */
 	tmp = 0;
@@ -263,7 +263,6 @@ _gnutls_sbase64_decode(char *data, size_t idata_size, uint8_t ** result)
 		tmp = decode(tmpres, datrev);
 		if (tmp < 0) {
 			gnutls_free((*result));
-			*result = NULL;
 			return tmp;
 		}
 
@@ -277,7 +276,6 @@ _gnutls_sbase64_decode(char *data, size_t idata_size, uint8_t ** result)
 		tmp = decode(tmpres, (uint8_t *) & data[i]);
 		if (tmp < 0) {
 			gnutls_free((*result));
-			*result = NULL;
 			return tmp;
 		}
 		memcpy(&(*result)[j], tmpres, tmp);
@@ -331,7 +329,7 @@ gnutls_srp_base64_encode(const gnutls_datum_t * data, char *result,
 }
 
 /**
- * gnutls_srp_base64_encode_alloc:
+ * gnutls_srp_base64_encode2:
  * @data: contains the raw data
  * @result: will hold the newly allocated encoded data
  *
@@ -348,7 +346,7 @@ gnutls_srp_base64_encode(const gnutls_datum_t * data, char *result,
  * Returns: 0 on success, or an error code.
  **/
 int
-gnutls_srp_base64_encode_alloc(const gnutls_datum_t * data,
+gnutls_srp_base64_encode2(const gnutls_datum_t * data,
 			       gnutls_datum_t * result)
 {
 	char *res;
@@ -413,7 +411,7 @@ gnutls_srp_base64_decode(const gnutls_datum_t * b64_data, char *result,
 }
 
 /**
- * gnutls_srp_base64_decode_alloc:
+ * gnutls_srp_base64_decode2:
  * @b64_data: contains the encoded data
  * @result: the place where decoded data lie
  *
@@ -429,7 +427,7 @@ gnutls_srp_base64_decode(const gnutls_datum_t * b64_data, char *result,
  * Returns: 0 on success, or an error code.
  **/
 int
-gnutls_srp_base64_decode_alloc(const gnutls_datum_t * b64_data,
+gnutls_srp_base64_decode2(const gnutls_datum_t * b64_data,
 			       gnutls_datum_t * result)
 {
 	uint8_t *ret;
