@@ -20,11 +20,11 @@
  *
  */
 
-/* 
+/*
  * This file holds all the buffering code used in gnutls.
  * The buffering code works as:
  *
- * RECORD LAYER: 
+ * RECORD LAYER:
  *  1. uses a buffer to hold data (application/handshake),
  *    we got but they were not requested, yet.
  *  (see gnutls_record_buffer_put(), gnutls_record_buffer_get_size() etc.)
@@ -32,7 +32,7 @@
  *  2. uses a buffer to hold data that were incomplete (ie the read/write
  *    was interrupted)
  *  (see _gnutls_io_read_buffered(), _gnutls_io_write_buffered() etc.)
- * 
+ *
  * HANDSHAKE LAYER:
  *  1. Uses buffer to hold the last received handshake message.
  *  (see _gnutls_handshake_hash_buffer_put() etc.)
@@ -67,12 +67,12 @@
  */
 void
 _gnutls_record_buffer_put(gnutls_session_t session,
-			  content_type_t type, gnutls_uint64 * seq,
+			  content_type_t type, uint64_t seq,
 			  mbuffer_st * bufel)
 {
 
 	bufel->type = type;
-	memcpy(&bufel->record_sequence, seq, sizeof(*seq));
+	bufel->record_sequence = seq;
 
 	_mbuffer_enqueue(&session->internals.record_buffer, bufel);
 	_gnutls_buffers_log("BUF[REC]: Inserted %d bytes of Data(%d)\n",
@@ -102,7 +102,7 @@ size_t gnutls_record_check_pending(gnutls_session_t session)
  * @session: is a #gnutls_session_t type.
  *
  * This function checks if there pending corked
- * data in the gnutls buffers --see gnutls_record_cork(). 
+ * data in the gnutls buffers --see gnutls_record_cork().
  *
  * Returns: Returns the size of the corked data or zero.
  *
@@ -155,7 +155,7 @@ _gnutls_record_buffer_get(content_type_t type,
 		length = msg.size;
 
 	if (seq)
-		memcpy(seq, bufel->record_sequence.i, 8);
+		_gnutls_write_uint64(bufel->record_sequence, seq);
 
 	memcpy(data, msg.data, length);
 	_mbuffer_head_remove_bytes(&session->internals.record_buffer,
@@ -517,13 +517,13 @@ _gnutls_writev(gnutls_session_t session, const giovec_t * giovec,
 	return i;
 }
 
-/* 
+/*
  * @ms: a pointer to the number of milliseconds to wait for data. Use zero or NULL for indefinite.
  *
  * This function is like recv(with MSG_PEEK). But it does not return -1 on error.
  * It does return gnutls_errno instead.
  * This function reads data from the socket and keeps them in a buffer, of up to
- * max_record_recv_size. 
+ * max_record_recv_size.
  *
  * This is not a general purpose function. It returns EXACTLY the data requested,
  * which are stored in a local (in the session) buffer.
@@ -741,10 +741,8 @@ int _gnutls_io_check_recv(gnutls_session_t session, unsigned int ms)
 	gnutls_transport_ptr_t fd = session->internals.transport_recv_ptr;
 	int ret = 0, err;
 
-	if (unlikely
-	    (session->internals.pull_timeout_func == gnutls_system_recv_timeout
-	     && session->internals.pull_func != system_read)) {
-		_gnutls_debug_log("The pull function has been replaced but not the pull timeout.");
+	if (NO_TIMEOUT_FUNC_SET(session)) {
+		_gnutls_debug_log("The pull function has been replaced but not the pull timeout.\n");
 		return gnutls_assert_val(GNUTLS_E_PULL_ERROR);
 	}
 
@@ -765,7 +763,7 @@ int _gnutls_io_check_recv(gnutls_session_t session, unsigned int ms)
 		return GNUTLS_E_TIMEDOUT;
 }
 
-/* HANDSHAKE buffers part 
+/* HANDSHAKE buffers part
  */
 
 /* This function writes the data that are left in the
@@ -794,9 +792,26 @@ ssize_t _gnutls_handshake_io_write_flush(gnutls_session_t session)
 	{
 		epoch = cur->epoch;
 
-		ret = _gnutls_send_int(session, cur->type,
-				       cur->htype,
-				       epoch, msg.data, msg.size, 0);
+		if (session->internals.h_read_func) {
+			record_parameters_st *params;
+
+			ret = _gnutls_epoch_get(session, epoch, &params);
+			if (ret < 0)
+				return gnutls_assert_val(ret);
+			ret = session->internals.h_read_func(session,
+							     params->write.level,
+							     cur->htype,
+							     msg.data,
+							     msg.size);
+			if (ret < 0)
+				return gnutls_assert_val(ret);
+
+			ret = msg.size;
+		} else {
+			ret = _gnutls_send_int(session, cur->type,
+					       cur->htype,
+					       epoch, msg.data, msg.size, 0);
+		}
 
 		if (ret >= 0) {
 			total += ret;
@@ -825,7 +840,7 @@ ssize_t _gnutls_handshake_io_write_flush(gnutls_session_t session)
 }
 
 
-/* This is a send function for the gnutls handshake 
+/* This is a send function for the gnutls handshake
  * protocol. Just makes sure that all data have been sent.
  *
  */
@@ -1175,7 +1190,7 @@ static int get_last_packet(gnutls_session_t session,
 	RETURN_DTLS_EAGAIN_OR_TIMEOUT(session, 0);
 }
 
-/* This is a receive function for the gnutls handshake 
+/* This is a receive function for the gnutls handshake
  * protocol. Makes sure that we have received all data.
  *
  * htype is the next handshake packet expected.
@@ -1305,7 +1320,7 @@ int _gnutls_parse_record_buffered_msgs(gnutls_session_t session)
 		handshake_buffer_st tmp;
 
 		do {
-			/* we now 
+			/* we now
 			 * 0. parse headers
 			 * 1. insert to handshake_recv_buffer
 			 * 2. sort handshake_recv_buffer on sequence numbers
@@ -1393,7 +1408,7 @@ int _gnutls_parse_record_buffered_msgs(gnutls_session_t session)
 	}
 }
 
-/* This is a receive function for the gnutls handshake 
+/* This is a receive function for the gnutls handshake
  * protocol. Makes sure that we have received all data.
  */
 ssize_t
@@ -1431,6 +1446,10 @@ _gnutls_handshake_io_recv_int(gnutls_session_t session,
 			return gnutls_assert_val(ret);
 	}
 
+	/* If handshake is handled manually, don't receive records from I/O */
+	if (session->internals.h_read_func)
+		return GNUTLS_E_AGAIN;
+
 	if (htype != (gnutls_handshake_description_t) -1) {
 		ret = handshake_remaining_time(session);
 		if (ret < 0)
@@ -1439,7 +1458,7 @@ _gnutls_handshake_io_recv_int(gnutls_session_t session,
 	}
 
 	do {
-		/* if we don't have a complete message waiting for us, try 
+		/* if we don't have a complete message waiting for us, try
 		 * receiving more */
 		ret =
 		    _gnutls_recv_in_buffers(session, GNUTLS_HANDSHAKE, htype,
