@@ -85,7 +85,11 @@ _gnutls_encrypt(gnutls_session_t session,
 		content_type_t type, record_parameters_st *params)
 {
 	gnutls_datum_t plaintext;
-	const version_entry_st *vers = get_version(session);
+	const version_entry_st *vers =
+		(session->internals.hsk_flags & HSK_EARLY_DATA_IN_FLIGHT) &&
+		!IS_SERVER(session) ?
+		session->internals.resumed_security_parameters.pversion :
+		get_version(session);
 	int ret;
 
 	plaintext.data = (uint8_t *) data;
