@@ -55,7 +55,7 @@ int _gnutls_cipher_exists(gnutls_cipher_algorithm_t cipher)
 	const gnutls_crypto_cipher_st *cc;
 	int ret;
 
-	if (is_cipher_algo_forbidden(cipher))
+	if (!is_cipher_algo_allowed(cipher))
 		return 0;
 
 	/* All the other ciphers are disabled on the back-end library.
@@ -101,6 +101,8 @@ _gnutls_cipher_init(cipher_hd_st *handle, const cipher_entry_st *e,
 		handle->auth = cc->auth;
 		handle->tag = cc->tag;
 		handle->setiv = cc->setiv;
+		handle->getiv = cc->getiv;
+		handle->setkey = cc->setkey;
 
 		/* if cc->init() returns GNUTLS_E_NEED_FALLBACK we
 		 * use the default ciphers */
@@ -108,8 +110,15 @@ _gnutls_cipher_init(cipher_hd_st *handle, const cipher_entry_st *e,
 		SR_FB(cc->setkey(handle->handle, key->data, key->size),
 		   cc_cleanup);
 		if (iv) {
-			if (unlikely(cc->setiv == NULL)) /* the API doesn't accept IV */
+			/* the API doesn't accept IV */
+			if (unlikely(cc->setiv == NULL)) {
+				if (cc->aead_encrypt) {
+					if (handle->handle)
+						handle->deinit(handle->handle);
+					goto fallback;
+				}
 				return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+			}
 			SR(cc->setiv(handle->handle, iv->data, iv->size),
 			   cc_cleanup);
 		}
@@ -126,6 +135,8 @@ _gnutls_cipher_init(cipher_hd_st *handle, const cipher_entry_st *e,
 	handle->auth = _gnutls_cipher_ops.auth;
 	handle->tag = _gnutls_cipher_ops.tag;
 	handle->setiv = _gnutls_cipher_ops.setiv;
+	handle->getiv = _gnutls_cipher_ops.getiv;
+	handle->setkey = _gnutls_cipher_ops.setkey;
 
 	/* otherwise use generic cipher interface
 	 */
@@ -216,6 +227,9 @@ int _gnutls_auth_cipher_init(auth_cipher_hd_st * handle,
 			gnutls_assert();
 			goto cleanup;
 		}
+#ifdef ENABLE_GOST
+		handle->continuous_mac = !!(me->flags & GNUTLS_MAC_FLAG_CONTINUOUS_MAC);
+#endif
 
 		handle->tag_size = _gnutls_mac_get_algo_len(me);
 	} else if (_gnutls_cipher_algo_is_aead(e)) {
@@ -420,14 +434,22 @@ int _gnutls_auth_cipher_tag(auth_cipher_hd_st * handle, void *tag,
 {
 	if (handle->is_mac) {
 #ifdef ENABLE_SSL3
-		int ret;
-
 		if (handle->ssl_hmac) {
-			ret =
+			int ret =
 			    _gnutls_mac_output_ssl3(&handle->mac.dig, tag);
 			if (ret < 0)
 				return gnutls_assert_val(ret);
 		} else
+#endif
+#ifdef ENABLE_GOST
+		/* draft-smyshlyaev-tls12-gost-suites section 4.1.2 */
+		if (handle->continuous_mac) {
+			mac_hd_st temp_mac;
+			int ret = _gnutls_mac_copy(&handle->mac.mac, &temp_mac);
+			if (ret < 0)
+				return gnutls_assert_val(ret);
+			_gnutls_mac_deinit(&temp_mac, tag);
+		} else
 #endif
 			_gnutls_mac_output(&handle->mac.mac, tag);
 	} else if (_gnutls_cipher_is_aead(&handle->cipher)) {
