@@ -118,6 +118,8 @@ const char
 		return "FINISHED";
 	case GNUTLS_HANDSHAKE_KEY_UPDATE:
 		return "KEY_UPDATE";
+	case GNUTLS_HANDSHAKE_COMPRESSED_CERTIFICATE_PKT:
+		return "COMPRESSED CERTIFICATE";
 	case GNUTLS_HANDSHAKE_SUPPLEMENTAL:
 		return "SUPPLEMENTAL";
 	case GNUTLS_HANDSHAKE_CERTIFICATE_STATUS:
