@@ -26,9 +26,9 @@
 #include "num.h"
 #include <ext/compress_certificate.h>
 
-/* Check whether certificate compression method is valid, ie. supported by gnutls
- */
-static inline int is_valid_method(gnutls_compression_method_t method)
+/* Check whether certificate compression method is valid, ie. supported by gnutls */
+static inline int
+is_valid_method(gnutls_compression_method_t method)
 {
 	switch (method) {
 #ifdef HAVE_LIBZ
@@ -48,12 +48,11 @@ static inline int is_valid_method(gnutls_compression_method_t method)
 	}
 }
 
-/* Converts compression algorithm number established in RFC8879 to internal compression method type
- */
-gnutls_compression_method_t
+/* Converts compression algorithm number established in RFC8879 to internal compression method type */
+gnutls_compression_method_t 
 _gnutls_compress_certificate_num2method(uint16_t num)
 {
-	switch (num) {
+	switch (num)  {
 	case 1:
 		return GNUTLS_COMP_ZLIB;
 	case 2:
@@ -62,12 +61,12 @@ _gnutls_compress_certificate_num2method(uint16_t num)
 		return GNUTLS_COMP_ZSTD;
 	default:
 		return GNUTLS_COMP_UNKNOWN;
-	}
+	 }
 }
 
-/* Converts compression method type to compression algorithm number established in RFC8879
- */
-int _gnutls_compress_certificate_method2num(gnutls_compression_method_t method)
+/* Converts compression method type to compression algorithm number established in RFC8879 */
+int 
+_gnutls_compress_certificate_method2num(gnutls_compression_method_t method)
 {
 	switch (method) {
 	case GNUTLS_COMP_ZLIB:
@@ -81,30 +80,6 @@ int _gnutls_compress_certificate_method2num(gnutls_compression_method_t method)
 	}
 }
 
-/* Returns 1 if the method is set as supported compression method for the session,
- * returns 0 otherwise
- */
-bool _gnutls_compress_certificate_is_method_enabled(
-	gnutls_session_t session, gnutls_compression_method_t method)
-{
-	int ret;
-	unsigned i;
-	compress_certificate_ext_st *priv;
-	gnutls_ext_priv_data_t epriv;
-
-	ret = _gnutls_hello_ext_get_priv(
-		session, GNUTLS_EXTENSION_COMPRESS_CERTIFICATE, &epriv);
-	if (ret < 0)
-		return false;
-	priv = epriv;
-
-	for (i = 0; i < priv->methods_len; ++i)
-		if (priv->methods[i] == method)
-			return true;
-
-	return false;
-}
-
 /**
  * gnutls_compress_certificate_get_selected_method:
  * @session: is a #gnutls_session_t type.
@@ -135,7 +110,7 @@ gnutls_compress_certificate_get_selected_method(gnutls_session_t session)
  * for a) requesting the compression of peer's certificate and b) selecting the
  * method to compress the local certificate before sending it to the peer.
  * The order of compression methods inside the list does matter as the method
- * that appears earlier in the list will be preferred before the later ones.
+ * that appears earlier in the list will be preffered before the later ones.
  * Note that even if you set the list of supported compression methods, the
  * compression might not be used if the peer does not support any of your chosen
  * compression methods.
@@ -155,16 +130,16 @@ gnutls_compress_certificate_get_selected_method(gnutls_session_t session)
  *
  * Since 3.7.4
  **/
-int gnutls_compress_certificate_set_methods(
-	gnutls_session_t session, const gnutls_compression_method_t *methods,
-	size_t methods_len)
+int
+gnutls_compress_certificate_set_methods(gnutls_session_t session,
+					const gnutls_compression_method_t * methods,
+					size_t methods_len)
 {
 	unsigned i;
 	compress_certificate_ext_st *priv;
 
 	if (methods == NULL || methods_len == 0) {
-		_gnutls_hello_ext_unset_priv(
-			session, GNUTLS_EXTENSION_COMPRESS_CERTIFICATE);
+		_gnutls_hello_ext_unset_priv(session, GNUTLS_EXTENSION_COMPRESS_CERTIFICATE);
 		return 0;
 	}
 
@@ -181,15 +156,15 @@ int gnutls_compress_certificate_set_methods(
 
 	priv->methods_len = methods_len;
 	memcpy(priv->methods, methods, methods_len * sizeof(*methods));
-	_gnutls_hello_ext_set_priv(session,
-				   GNUTLS_EXTENSION_COMPRESS_CERTIFICATE, priv);
+	_gnutls_hello_ext_set_priv(session, GNUTLS_EXTENSION_COMPRESS_CERTIFICATE, priv);
 
 	return 0;
 }
 
-int _gnutls_compress_certificate_recv_params(gnutls_session_t session,
-					     const uint8_t *data,
-					     size_t data_size)
+int
+_gnutls_compress_certificate_recv_params(gnutls_session_t session,
+					 const uint8_t * data,
+					 size_t data_size)
 {
 	int ret;
 	unsigned i, j;
@@ -197,33 +172,31 @@ int _gnutls_compress_certificate_recv_params(gnutls_session_t session,
 	uint8_t bytes_len;
 	size_t methods_len;
 	gnutls_compression_method_t methods[MAX_COMPRESS_CERTIFICATE_METHODS];
-	gnutls_compression_method_t method;
+	gnutls_compression_method_t method = GNUTLS_COMP_UNKNOWN;
 	compress_certificate_ext_st *priv;
 	gnutls_ext_priv_data_t epriv;
 
-	ret = _gnutls_hello_ext_get_priv(
-		session, GNUTLS_EXTENSION_COMPRESS_CERTIFICATE, &epriv);
+	ret = _gnutls_hello_ext_get_priv(session, GNUTLS_EXTENSION_COMPRESS_CERTIFICATE, &epriv);
 	if (ret < 0)
 		return 0;
 	priv = epriv;
 
 	DECR_LEN(data_size, 1);
 	bytes_len = *data;
-
+	
 	if (bytes_len < 2 || bytes_len > 254 || bytes_len % 2 == 1)
-		return gnutls_assert_val(GNUTLS_E_UNEXPECTED_PACKET_LENGTH);
+		return gnutls_assert_val(GNUTLS_E_RECEIVED_ILLEGAL_PARAMETER);
 
 	DECR_LEN(data_size, bytes_len);
+	methods_len = bytes_len / 2;
 
-	methods_len = 0;
-	for (i = 0; i < bytes_len / 2; ++i) {
+	for (i = 0; i < methods_len; ++i) {
 		num = _gnutls_read_uint16(data + i + i + 1);
-		method = _gnutls_compress_certificate_num2method(num);
-		if (method != GNUTLS_COMP_UNKNOWN)
-			methods[methods_len++] = method;
+		methods[i] = _gnutls_compress_certificate_num2method(num);
+		if (methods[i] == GNUTLS_COMP_UNKNOWN)
+			return gnutls_assert_val(GNUTLS_E_RECEIVED_ILLEGAL_PARAMETER);
 	}
 
-	method = GNUTLS_COMP_UNKNOWN;
 	for (i = 0; i < methods_len; ++i)
 		for (j = 0; j < priv->methods_len; ++j)
 			if (methods[i] == priv->methods[j]) {
@@ -236,8 +209,9 @@ endloop:
 	return 0;
 }
 
-int _gnutls_compress_certificate_send_params(gnutls_session_t session,
-					     gnutls_buffer_st *data)
+int
+_gnutls_compress_certificate_send_params(gnutls_session_t session,
+					 gnutls_buffer_st * data)
 {
 	int ret, num;
 	unsigned i;
@@ -246,8 +220,7 @@ int _gnutls_compress_certificate_send_params(gnutls_session_t session,
 	compress_certificate_ext_st *priv;
 	gnutls_ext_priv_data_t epriv;
 
-	ret = _gnutls_hello_ext_get_priv(
-		session, GNUTLS_EXTENSION_COMPRESS_CERTIFICATE, &epriv);
+	ret = _gnutls_hello_ext_get_priv(session, GNUTLS_EXTENSION_COMPRESS_CERTIFICATE, &epriv);
 	if (ret < 0)
 		return 0;
 	priv = epriv;
@@ -262,8 +235,6 @@ int _gnutls_compress_certificate_send_params(gnutls_session_t session,
 	if (ret < 0)
 		return gnutls_assert_val(ret);
 
-	session->internals.hsk_flags |= HSK_COMP_CRT_REQ_SENT;
-
 	return bytes_len + 1;
 }
 
@@ -273,7 +244,7 @@ const hello_ext_entry_st ext_mod_compress_certificate = {
 	.gid = GNUTLS_EXTENSION_COMPRESS_CERTIFICATE,
 	.client_parse_point = GNUTLS_EXT_TLS,
 	.server_parse_point = GNUTLS_EXT_TLS,
-	.validity = GNUTLS_EXT_FLAG_TLS | GNUTLS_EXT_FLAG_DTLS |
+	.validity = GNUTLS_EXT_FLAG_TLS | GNUTLS_EXT_FLAG_DTLS | 
 		    GNUTLS_EXT_FLAG_CLIENT_HELLO,
 	.recv_func = _gnutls_compress_certificate_recv_params,
 	.send_func = _gnutls_compress_certificate_send_params,
