@@ -46,7 +46,8 @@ const hello_ext_entry_st ext_mod_srp = {
 	.name = "SRP",
 	.tls_id = 12,
 	.gid = GNUTLS_EXTENSION_SRP,
-	.parse_type = GNUTLS_EXT_TLS,
+	.client_parse_point = GNUTLS_EXT_TLS,
+	.server_parse_point = GNUTLS_EXT_TLS,
 	.validity = GNUTLS_EXT_FLAG_TLS | GNUTLS_EXT_FLAG_DTLS | GNUTLS_EXT_FLAG_CLIENT_HELLO,
 	.recv_func = _gnutls_srp_recv_params,
 	.send_func = _gnutls_srp_send_params,
@@ -59,10 +60,9 @@ const hello_ext_entry_st ext_mod_srp = {
 
 static int
 _gnutls_srp_recv_params(gnutls_session_t session, const uint8_t * data,
-			size_t _data_size)
+			size_t data_size)
 {
 	uint8_t len;
-	ssize_t data_size = _data_size;
 	gnutls_ext_priv_data_t epriv;
 	srp_ext_st *priv;
 
@@ -73,11 +73,6 @@ _gnutls_srp_recv_params(gnutls_session_t session, const uint8_t * data,
 			len = data[0];
 			DECR_LEN(data_size, len);
 
-			if (MAX_USERNAME_SIZE < len) {
-				gnutls_assert();
-				return GNUTLS_E_ILLEGAL_SRP_USERNAME;
-			}
-
 			priv = gnutls_calloc(1, sizeof(*priv));
 			if (priv == NULL) {
 				gnutls_assert();
