@@ -84,34 +84,19 @@ int _gnutls13_handshake_client(gnutls_session_t session)
 	case STATE99:
 	case STATE100:
 #ifdef TLS13_APPENDIX_D4
-		/* We send it before keys are generated. That works because CCS
-		 * is always being cached and queued and not being sent directly */
-		ret = _gnutls_send_change_cipher_spec(session, AGAIN(STATE100));
-		STATE = STATE100;
-		IMED_RET("send change cipher spec", ret, 0);
+		if (session->internals.priorities->tls13_compat_mode &&
+		    /* Key change is indicated by sending an EndOfEarlyData below */
+		    !(session->internals.hsk_flags & HSK_EARLY_DATA_IN_FLIGHT)) {
+			/* We send it before keys are generated. That works because CCS
+			 * is always being cached and queued and not being sent directly */
+			ret = _gnutls_send_change_cipher_spec(session, AGAIN(STATE100));
+			STATE = STATE100;
+			IMED_RET("send change cipher spec", ret, 0);
+		}
 #endif
 		FALLTHROUGH;
 	case STATE101:
-		/* Note that we check IN_FLIGHT, not ACCEPTED
-		 * here. This is because the client sends early data
-		 * speculatively. */
-		if (session->internals.hsk_flags & HSK_EARLY_DATA_IN_FLIGHT) {
-			ret = _tls13_write_connection_state_init(session, STAGE_EARLY);
-			if (ret == 0) {
-				_gnutls_epoch_bump(session);
-				ret = _gnutls_epoch_dup(session, EPOCH_WRITE_CURRENT);
-			}
-			STATE = STATE101;
-			IMED_RET_FATAL("set early traffic keys", ret, 0);
-		}
-		FALLTHROUGH;
-	case STATE102:
-		ret = _gnutls13_send_early_data(session);
-		STATE = STATE102;
-		IMED_RET("send early data", ret, 0);
-		FALLTHROUGH;
-	case STATE103:
-		STATE = STATE103;
+		STATE = STATE101;
 		ret = generate_hs_traffic_keys(session);
 		/* Note that we check IN_FLIGHT, not ACCEPTED
 		 * here. This is because the client sends early data
@@ -123,40 +108,40 @@ int _gnutls13_handshake_client(gnutls_session_t session)
 			ret = _tls13_connection_state_init(session, STAGE_HS);
 		IMED_RET_FATAL("set hs traffic keys", ret, 0);
 		FALLTHROUGH;
-	case STATE104:
+	case STATE102:
 		ret = _gnutls13_recv_encrypted_extensions(session);
-		STATE = STATE104;
+		STATE = STATE102;
 		IMED_RET("recv encrypted extensions", ret, 0);
 		FALLTHROUGH;
-	case STATE105:
+	case STATE103:
 		ret = _gnutls13_recv_certificate_request(session);
-		STATE = STATE105;
+		STATE = STATE103;
 		IMED_RET("recv certificate request", ret, 0);
 		FALLTHROUGH;
-	case STATE106:
+	case STATE104:
 		ret = _gnutls13_recv_certificate(session);
-		STATE = STATE106;
+		STATE = STATE104;
 		IMED_RET("recv certificate", ret, 0);
 		FALLTHROUGH;
-	case STATE107:
+	case STATE105:
 		ret = _gnutls13_recv_certificate_verify(session);
-		STATE = STATE107;
+		STATE = STATE105;
 		IMED_RET("recv server certificate verify", ret, 0);
 		FALLTHROUGH;
-	case STATE108:
+	case STATE106:
 		ret = _gnutls_run_verify_callback(session, GNUTLS_CLIENT);
-		STATE = STATE108;
+		STATE = STATE106;
 		if (ret < 0)
 			return gnutls_assert_val(ret);
 		FALLTHROUGH;
-	case STATE109:
+	case STATE107:
 		ret = _gnutls13_recv_finished(session);
-		STATE = STATE109;
+		STATE = STATE107;
 		IMED_RET("recv finished", ret, 0);
 		FALLTHROUGH;
-	case STATE110:
-		ret = _gnutls13_send_end_of_early_data(session, AGAIN(STATE110));
-		STATE = STATE110;
+	case STATE108:
+		ret = _gnutls13_send_end_of_early_data(session, AGAIN(STATE108));
+		STATE = STATE108;
 		IMED_RET("send end of early data", ret, 0);
 
 		/* Note that we check IN_FLIGHT, not ACCEPTED
@@ -168,23 +153,23 @@ int _gnutls13_handshake_client(gnutls_session_t session)
 			IMED_RET_FATAL("set hs traffic key after sending early data", ret, 0);
 		}
 		FALLTHROUGH;
-	case STATE111:
-		ret = _gnutls13_send_certificate(session, AGAIN(STATE111));
-		STATE = STATE111;
+	case STATE109:
+		ret = _gnutls13_send_certificate(session, AGAIN(STATE109));
+		STATE = STATE109;
 		IMED_RET("send certificate", ret, 0);
 		FALLTHROUGH;
-	case STATE112:
-		ret = _gnutls13_send_certificate_verify(session, AGAIN(STATE112));
-		STATE = STATE112;
+	case STATE110:
+		ret = _gnutls13_send_certificate_verify(session, AGAIN(STATE110));
+		STATE = STATE110;
 		IMED_RET("send certificate verify", ret, 0);
 		FALLTHROUGH;
-	case STATE113:
-		ret = _gnutls13_send_finished(session, AGAIN(STATE113));
-		STATE = STATE113;
+	case STATE111:
+		ret = _gnutls13_send_finished(session, AGAIN(STATE111));
+		STATE = STATE111;
 		IMED_RET("send finished", ret, 0);
 		FALLTHROUGH;
-	case STATE114:
-		STATE = STATE114;
+	case STATE112:
+		STATE = STATE112;
 
 		ret =
 		    generate_ap_traffic_keys(session);
@@ -385,9 +370,11 @@ int _gnutls13_handshake_server(gnutls_session_t session)
 		FALLTHROUGH;
 	case STATE92:
 #ifdef TLS13_APPENDIX_D4
-		ret = _gnutls_send_change_cipher_spec(session, AGAIN(STATE92));
-		STATE = STATE92;
-		IMED_RET("send change cipher spec", ret, 0);
+		if (session->internals.priorities->tls13_compat_mode) {
+			ret = _gnutls_send_change_cipher_spec(session, AGAIN(STATE92));
+			STATE = STATE92;
+			IMED_RET("send change cipher spec", ret, 0);
+		}
 #endif
 		FALLTHROUGH;
 	case STATE93:
@@ -416,7 +403,8 @@ int _gnutls13_handshake_server(gnutls_session_t session)
 #ifdef TLS13_APPENDIX_D4
 		/* don't send CCS twice: when HRR has already been
 		 * sent, CCS should have followed it (see above) */
-		if (!(session->internals.hsk_flags & HSK_HRR_SENT)) {
+		if (session->internals.priorities->tls13_compat_mode &&
+		    !(session->internals.hsk_flags & HSK_HRR_SENT)) {
 			ret = _gnutls_send_change_cipher_spec(session, AGAIN(STATE100));
 			STATE = STATE100;
 			IMED_RET("send change cipher spec", ret, 0);
@@ -425,22 +413,13 @@ int _gnutls13_handshake_server(gnutls_session_t session)
 		FALLTHROUGH;
 	case STATE101:
 		STATE = STATE101;
+
+		ret = generate_hs_traffic_keys(session);
+		IMED_RET_FATAL("generate hs traffic keys", ret, 0);
+
 		if (session->internals.hsk_flags & HSK_EARLY_DATA_ACCEPTED) {
-			ret = _tls13_read_connection_state_init(session, STAGE_EARLY);
-			if (ret == 0) {
-				_gnutls_epoch_bump(session);
-				ret = _gnutls_epoch_dup(session, EPOCH_READ_CURRENT);
-			}
-			IMED_RET_FATAL("set early traffic keys", ret, 0);
-
-			ret = generate_hs_traffic_keys(session);
-			IMED_RET_FATAL("generate hs traffic keys", ret, 0);
-
 			ret = _tls13_write_connection_state_init(session, STAGE_HS);
 		} else {
-			ret = generate_hs_traffic_keys(session);
-			IMED_RET_FATAL("generate hs traffic keys", ret, 0);
-
 			ret = _tls13_connection_state_init(session, STAGE_HS);
 		}
 		IMED_RET_FATAL("set hs traffic keys", ret, 0);
@@ -510,9 +489,11 @@ int _gnutls13_handshake_server(gnutls_session_t session)
 			_gnutls_set_resumed_parameters(session);
 
 		if (session->internals.hsk_flags & HSK_EARLY_START_USED) {
-			if (!(session->internals.flags & GNUTLS_NO_AUTO_SEND_TICKET))
+			if (!(session->internals.flags & GNUTLS_NO_AUTO_SEND_TICKET) &&
+			    _gnutls13_can_send_session_ticket(session)) {
 				ret = _gnutls13_send_session_ticket(session, TLS13_TICKETS_TO_SEND,
 								    AGAIN(STATE109));
+			}
 
 			STATE = STATE109;
 			IMED_RET("send session ticket", ret, 0);
@@ -567,7 +548,8 @@ int _gnutls13_handshake_server(gnutls_session_t session)
 		FALLTHROUGH;
 	case STATE115:
 		if (!(session->internals.hsk_flags & (HSK_TLS13_TICKET_SENT|HSK_EARLY_START_USED)) &&
-		    !(session->internals.flags & GNUTLS_NO_AUTO_SEND_TICKET)) {
+		    !(session->internals.flags & GNUTLS_NO_AUTO_SEND_TICKET) &&
+		    _gnutls13_can_send_session_ticket(session)) {
 			ret = _gnutls13_send_session_ticket(session, TLS13_TICKETS_TO_SEND,
 							    AGAIN(STATE115));
 			STATE = STATE115;
