@@ -16,7 +16,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>
  */
 
 #include "gnutls_int.h"
@@ -28,6 +28,7 @@
 #include <pkcs11_int.h>
 #include <p11-kit/p11-kit.h>
 #include "pkcs11x.h"
+#include "intprops.h"
 
 struct find_ext_data_st {
 	/* in */
@@ -217,10 +218,17 @@ find_ext_cb(struct ck_function_list *module, struct pkcs11_session_info *sinfo,
 		rv = pkcs11_get_attribute_avalue(sinfo->module, sinfo->pks, obj, CKA_VALUE, &ext);
 		if (rv == CKR_OK) {
 
-			find_data->exts = gnutls_realloc_fast(find_data->exts, (1+find_data->exts_size)*sizeof(find_data->exts[0]));
+			if (unlikely(INT_ADD_OVERFLOW(find_data->exts_size, 1))) {
+				ret = gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
+				goto cleanup;
+			}
+
+			find_data->exts =
+				_gnutls_reallocarray_fast(find_data->exts,
+							  find_data->exts_size + 1,
+							  sizeof(find_data->exts[0]));
 			if (find_data->exts == NULL) {
-				gnutls_assert();
-				ret = pkcs11_rv_to_err(rv);
+				ret = gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
 				goto cleanup;
 			}
 
@@ -277,7 +285,7 @@ gnutls_pkcs11_obj_get_exts(gnutls_pkcs11_obj_t obj,
 		spki.data = obj->raw.data;
 		spki.size = obj->raw.size;
 	} else {
-		ret = x509_raw_crt_to_raw_pubkey(&obj->raw, &spki);
+		ret = _gnutls_x509_raw_crt_to_raw_pubkey(&obj->raw, &spki);
 		if (ret < 0)
 			return gnutls_assert_val(ret);
 		deinit_spki = 1;
