@@ -54,6 +54,9 @@
 #include <random.h>
 #include <xsize.h>
 #include "locks.h"
+#include "system/ktls.h"
+#include <intprops.h>
+
 
 struct tls_record_st {
 	uint16_t header_size;
@@ -288,7 +291,8 @@ int gnutls_bye(gnutls_session_t session, gnutls_close_request_t how)
 
 	switch (BYE_STATE) {
 	case BYE_STATE0:
-		ret = _gnutls_io_write_flush(session);
+		if (!IS_KTLS_ENABLED(session, GNUTLS_KTLS_SEND))
+			ret = _gnutls_io_write_flush(session);
 		BYE_STATE = BYE_STATE0;
 		if (ret < 0) {
 			gnutls_assert();
@@ -296,9 +300,8 @@ int gnutls_bye(gnutls_session_t session, gnutls_close_request_t how)
 		}
 		FALLTHROUGH;
 	case BYE_STATE1:
-		ret =
-		    gnutls_alert_send(session, GNUTLS_AL_WARNING,
-				      GNUTLS_A_CLOSE_NOTIFY);
+		ret = gnutls_alert_send(session, GNUTLS_AL_WARNING,
+				GNUTLS_A_CLOSE_NOTIFY);
 		BYE_STATE = BYE_STATE1;
 		if (ret < 0) {
 			gnutls_assert();
@@ -308,14 +311,22 @@ int gnutls_bye(gnutls_session_t session, gnutls_close_request_t how)
 	case BYE_STATE2:
 		BYE_STATE = BYE_STATE2;
 		if (how == GNUTLS_SHUT_RDWR) {
-			do {
-				ret =
-				    _gnutls_recv_int(session, GNUTLS_ALERT,
-						     NULL, 0, NULL,
-						     session->internals.
-						     record_timeout_ms);
+			if (IS_KTLS_ENABLED(session, GNUTLS_KTLS_SEND)){
+				do {
+					ret = _gnutls_ktls_recv_int(session,
+							GNUTLS_ALERT, NULL, 0);
+				}
+				while (ret == GNUTLS_E_GOT_APPLICATION_DATA);
+			} else {
+				do {
+					ret =
+						_gnutls_recv_int(session, GNUTLS_ALERT,
+								 NULL, 0, NULL,
+								 session->internals.
+								 record_timeout_ms);
+				}
+				while (ret == GNUTLS_E_GOT_APPLICATION_DATA);
 			}
-			while (ret == GNUTLS_E_GOT_APPLICATION_DATA);
 
 			if (ret >= 0)
 				session->internals.may_not_read = 1;
@@ -2026,9 +2037,13 @@ gnutls_record_send2(gnutls_session_t session, const void *data,
 
 	switch(session->internals.rsend_state) {
 		case RECORD_SEND_NORMAL:
-			return _gnutls_send_tlen_int(session, GNUTLS_APPLICATION_DATA,
-						     -1, EPOCH_WRITE_CURRENT, data,
-						     data_size, pad, MBUFFER_FLUSH);
+			if (IS_KTLS_ENABLED(session, GNUTLS_KTLS_SEND)) {
+				return _gnutls_ktls_send(session, data, data_size);
+			} else {
+				return _gnutls_send_tlen_int(session, GNUTLS_APPLICATION_DATA,
+								 -1, EPOCH_WRITE_CURRENT, data,
+								 data_size, pad, MBUFFER_FLUSH);
+			}
 		case RECORD_SEND_CORKED:
 		case RECORD_SEND_CORKED_TO_KU:
 			return append_data_to_corked(session, data, data_size);
@@ -2098,6 +2113,10 @@ ssize_t gnutls_record_send_early_data(gnutls_session_t session,
 	if (session->security_parameters.entity != GNUTLS_CLIENT)
 		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
 
+	if (data_size == 0) {
+		return 0;
+	}
+
 	if (xsum(session->internals.
 		 early_data_presend_buffer.length,
 		 data_size) >
@@ -2111,6 +2130,102 @@ ssize_t gnutls_record_send_early_data(gnutls_session_t session,
 	if (ret < 0)
 		return gnutls_assert_val(ret);
 
+	session->internals.flags |= GNUTLS_ENABLE_EARLY_DATA;
+
+	return ret;
+}
+
+/**
+ * gnutls_record_send_file:
+ * @session: is a #gnutls_session_t type.
+ * @fd: file descriptor from which to read data.
+ * @offset: Is relative to file offset, denotes the starting location for
+ *          reading.  after function returns, it point to position following
+ *          last read byte.
+ * @count: is the length of the data in bytes to be read from file and send.
+ *
+ * This function sends data from @fd. If KTLS (kernel TLS) is enabled, it will
+ * use the sendfile() system call to avoid overhead of copying data between user
+ * space and the kernel. Otherwise, this functionality is merely emulated by
+ * calling read() and gnutls_record_send(). If this implementation is
+ * suboptimal, check whether KTLS is enabled using
+ * gnutls_transport_is_ktls_enabled().
+ *
+ * If @offset is NULL then file offset is incremented by number of bytes send,
+ * otherwise file offset remains unchanged.
+ *
+ * Returns: The number of bytes sent, or a negative error code.
+ **/
+ssize_t gnutls_record_send_file(gnutls_session_t session, int fd,
+				off_t *offset, size_t count)
+{
+	ssize_t ret;
+	size_t buf_len;
+	size_t sent = 0;
+	uint8_t *buf;
+	off_t saved_offset = 0;
+
+	if (IS_KTLS_ENABLED(session, GNUTLS_KTLS_SEND)) {
+		return _gnutls_ktls_send_file(session, fd, offset, count);
+	}
+
+	if (offset != NULL) {
+		saved_offset = lseek(fd, 0, SEEK_CUR);
+		if (saved_offset == (off_t)-1) {
+			return GNUTLS_E_FILE_ERROR;
+		}
+		if (lseek(fd, *offset, SEEK_CUR) == -1) {
+			return GNUTLS_E_FILE_ERROR;
+		}
+	}
+
+	buf_len = MIN(count, MAX(max_record_send_size(session, NULL), 512));
+
+	buf = gnutls_malloc(buf_len);
+	if (buf == NULL) {
+		gnutls_assert();
+		ret = GNUTLS_E_MEMORY_ERROR;
+		goto end;
+	}
+
+	while (sent < count) {
+		ret = read(fd, buf, MIN(buf_len, count - sent));
+		if (ret == 0) {
+			break;
+		} else if (ret == -1){
+			if (errno == EAGAIN) {
+				ret = GNUTLS_E_AGAIN;
+				goto end;
+			}
+			ret = GNUTLS_E_FILE_ERROR;
+			goto end;
+		}
+
+		ret = gnutls_record_send(session, buf, ret);
+		if (ret < 0) {
+			goto end;
+		}
+		if (INT_ADD_OVERFLOW(sent, ret)) {
+			gnutls_assert();
+			ret = GNUTLS_E_RECORD_OVERFLOW;
+			goto end;
+		}
+		sent += ret;
+	}
+
+	ret = sent;
+
+  end:
+	if (offset != NULL){
+		if (likely(!INT_ADD_OVERFLOW(*offset, sent))) {
+			*offset += sent;
+		} else {
+			gnutls_assert();
+			ret = GNUTLS_E_RECORD_OVERFLOW;
+		}
+		lseek(fd, saved_offset, SEEK_SET);
+	}
+	gnutls_free(buf);
 	return ret;
 }
 
@@ -2120,7 +2235,7 @@ ssize_t gnutls_record_send_early_data(gnutls_session_t session,
  * @data: the buffer that the data will be read into
  * @data_size: the number of requested bytes
  *
- * This function can be used by a searver to retrieve data sent early
+ * This function can be used by a server to retrieve data sent early
  * in the handshake processes when resuming a session.  This is used
  * to implement a zero-roundtrip (0-RTT) mode.  It has the same
  * semantics as gnutls_record_recv().
@@ -2287,9 +2402,13 @@ gnutls_record_recv(gnutls_session_t session, void *data, size_t data_size)
 			return gnutls_assert_val(GNUTLS_E_UNAVAILABLE_DURING_HANDSHAKE);
 	}
 
-	return _gnutls_recv_int(session, GNUTLS_APPLICATION_DATA,
-				data, data_size, NULL,
-				session->internals.record_timeout_ms);
+	if (IS_KTLS_ENABLED(session, GNUTLS_KTLS_RECV)) {
+		return _gnutls_ktls_recv(session, data, data_size);
+	} else {
+		return _gnutls_recv_int(session, GNUTLS_APPLICATION_DATA,
+					data, data_size, NULL,
+					session->internals.record_timeout_ms);
+	}
 }
 
 /**
