@@ -21,8 +21,8 @@
  *
  */
 
-#include "config.h"
-#include "system.h"
+#include <config.h>
+#include <system.h>
 #include "gnutls_int.h"
 #include "errors.h"
 
@@ -57,7 +57,7 @@ static int gnutls_system_mutex_init(void **priv)
 
 static int gnutls_system_mutex_deinit(void **priv)
 {
-	if (glthread_lock_destroy((gl_lock_t *)*priv)) {
+	if (glthread_lock_destroy((gl_lock_t *) * priv)) {
 		return gnutls_assert_val(GNUTLS_E_LOCKING_ERROR);
 	}
 	free(*priv);
@@ -66,7 +66,7 @@ static int gnutls_system_mutex_deinit(void **priv)
 
 static int gnutls_system_mutex_lock(void **priv)
 {
-	if (glthread_lock_lock((gl_lock_t *)*priv)) {
+	if (glthread_lock_lock((gl_lock_t *) * priv)) {
 		return gnutls_assert_val(GNUTLS_E_LOCKING_ERROR);
 	}
 	return 0;
@@ -74,7 +74,7 @@ static int gnutls_system_mutex_lock(void **priv)
 
 static int gnutls_system_mutex_unlock(void **priv)
 {
-	if (glthread_lock_unlock((gl_lock_t *)*priv)) {
+	if (glthread_lock_unlock((gl_lock_t *) * priv)) {
 		return gnutls_assert_val(GNUTLS_E_LOCKING_ERROR);
 	}
 	return 0;
@@ -84,3 +84,4 @@ mutex_init_func gnutls_mutex_init = gnutls_system_mutex_init;
 mutex_deinit_func gnutls_mutex_deinit = gnutls_system_mutex_deinit;
 mutex_lock_func gnutls_mutex_lock = gnutls_system_mutex_lock;
 mutex_unlock_func gnutls_mutex_unlock = gnutls_system_mutex_unlock;
+
