@@ -97,11 +97,15 @@
 #define SIG_RSA_SHA3_384_OID "2.16.840.1.101.3.4.3.15"
 #define SIG_RSA_SHA3_512_OID "2.16.840.1.101.3.4.3.16"
 
+#define ECDH_X25519_OID "1.3.101.110"
+#define ECDH_X448_OID "1.3.101.111"
+
 #define SIG_EDDSA_SHA512_OID "1.3.101.112"
 #define SIG_ED448_OID "1.3.101.113"
 
 #define XMPP_OID "1.3.6.1.5.5.7.8.5"
 #define KRB5_PRINCIPAL_OID "1.3.6.1.5.2.2"
+#define MSUSER_PRINCIPAL_NAME_OID "1.3.6.1.4.1.311.20.2.3"
 #define PKIX1_RSA_PSS_MGF1_OID "1.2.840.113549.1.1.8"
 
 #define GOST28147_89_OID "1.2.643.2.2.21"
@@ -126,10 +130,12 @@ struct oid_to_string {
 
 const struct oid_to_string *_gnutls_oid_get_entry(const struct oid_to_string *ots, const char *oid);
 
-int _gnutls_x509_set_time(ASN1_TYPE c2, const char *where, time_t tim,
+const char *_gnutls_oid_get_asn_desc(const char *oid);
+
+int _gnutls_x509_set_time(asn1_node c2, const char *where, time_t tim,
 			  int force_general);
 int
-_gnutls_x509_set_raw_time(ASN1_TYPE c2, const char *where, time_t tim);
+_gnutls_x509_set_raw_time(asn1_node c2, const char *where, time_t tim);
 
 int _gnutls_x509_decode_string(unsigned int etype,
 			       const uint8_t * der, size_t der_size,
@@ -144,20 +150,20 @@ int _gnutls_x509_dn_to_string(const char *OID, void *value,
 			      int value_size, gnutls_datum_t * out);
 const char *_gnutls_ldap_string_to_oid(const char *str, unsigned str_len);
 
-time_t _gnutls_x509_get_time(ASN1_TYPE c2, const char *when, int general);
+time_t _gnutls_x509_get_time(asn1_node c2, const char *when, int general);
 
 gnutls_x509_subject_alt_name_t _gnutls_x509_san_find_type(char *str_type);
 
-int _gnutls_x509_der_encode_and_copy(ASN1_TYPE src, const char *src_name,
-				     ASN1_TYPE dest, const char *dest_name,
+int _gnutls_x509_der_encode_and_copy(asn1_node src, const char *src_name,
+				     asn1_node dest, const char *dest_name,
 				     int str);
-int _gnutls_x509_der_encode(ASN1_TYPE src, const char *src_name,
+int _gnutls_x509_der_encode(asn1_node src, const char *src_name,
 			    gnutls_datum_t * res, int str);
 
 #define _gnutls_x509_export_int(asn1, format, header, out, out_size) \
   _gnutls_x509_export_int_named(asn1, "", format, header, out, out_size)
 
-int _gnutls_x509_export_int_named(ASN1_TYPE asn1_data, const char *name,
+int _gnutls_x509_export_int_named(asn1_node asn1_data, const char *name,
 				  gnutls_x509_crt_fmt_t format,
 				  const char *pem_header,
 				  unsigned char *output_data,
@@ -165,59 +171,59 @@ int _gnutls_x509_export_int_named(ASN1_TYPE asn1_data, const char *name,
 
 #define _gnutls_x509_export_int2(asn1, format, header, out) \
   _gnutls_x509_export_int_named2(asn1, "", format, header, out)
-int _gnutls_x509_export_int_named2(ASN1_TYPE asn1_data, const char *name,
+int _gnutls_x509_export_int_named2(asn1_node asn1_data, const char *name,
 				   gnutls_x509_crt_fmt_t format,
 				   const char *pem_header,
 				   gnutls_datum_t * out);
 
-int _gnutls_x509_read_value(ASN1_TYPE c, const char *root,
+int _gnutls_x509_read_value(asn1_node c, const char *root,
 			    gnutls_datum_t * ret);
-int _gnutls_x509_read_null_value(ASN1_TYPE c, const char *root,
+int _gnutls_x509_read_null_value(asn1_node c, const char *root,
 			    gnutls_datum_t * ret);
-int _gnutls_x509_read_string(ASN1_TYPE c, const char *root,
+int _gnutls_x509_read_string(asn1_node c, const char *root,
 			     gnutls_datum_t * ret, unsigned int etype,
 			     unsigned allow_ber);
-int _gnutls_x509_write_value(ASN1_TYPE c, const char *root,
+int _gnutls_x509_write_value(asn1_node c, const char *root,
 			     const gnutls_datum_t * data);
 
-int _gnutls_x509_write_string(ASN1_TYPE c, const char *root,
+int _gnutls_x509_write_string(asn1_node c, const char *root,
 			      const gnutls_datum_t * data,
 			      unsigned int etype);
 
 int _gnutls_x509_encode_and_write_attribute(const char *given_oid,
-					    ASN1_TYPE asn1_struct,
+					    asn1_node asn1_struct,
 					    const char *where,
 					    const void *data,
 					    int sizeof_data, int multi);
-int _gnutls_x509_decode_and_read_attribute(ASN1_TYPE asn1_struct,
+int _gnutls_x509_decode_and_read_attribute(asn1_node asn1_struct,
 					   const char *where, char *oid,
 					   int oid_size,
 					   gnutls_datum_t * value,
 					   int multi, int octet);
 
-int _gnutls_x509_get_pk_algorithm(ASN1_TYPE src, const char *src_name,
+int _gnutls_x509_get_pk_algorithm(asn1_node src, const char *src_name,
 				  gnutls_ecc_curve_t *curve,
 				  unsigned int *bits);
 
 int
-_gnutls_x509_get_signature_algorithm(ASN1_TYPE src, const char *src_name);
+_gnutls_x509_get_signature_algorithm(asn1_node src, const char *src_name);
 
-int _gnutls_x509_encode_and_copy_PKI_params(ASN1_TYPE dst,
+int _gnutls_x509_encode_and_copy_PKI_params(asn1_node dst,
 					    const char *dst_name,
 					    const gnutls_pk_params_st * params);
 int _gnutls_x509_encode_PKI_params(gnutls_datum_t * der,
 				   const gnutls_pk_params_st * params);
-int _gnutls_asn1_copy_node(ASN1_TYPE * dst, const char *dst_name,
-			   ASN1_TYPE src, const char *src_name);
+int _gnutls_asn1_copy_node(asn1_node * dst, const char *dst_name,
+			   asn1_node src, const char *src_name);
 
-int _gnutls_x509_get_signed_data(ASN1_TYPE src, const gnutls_datum_t *der,
+int _gnutls_x509_get_signed_data(asn1_node src, const gnutls_datum_t *der,
 				 const char *src_name,
 				 gnutls_datum_t * signed_data);
-int _gnutls_x509_get_signature(ASN1_TYPE src, const char *src_name,
+int _gnutls_x509_get_signature(asn1_node src, const char *src_name,
 			       gnutls_datum_t * signature);
 
 
-int _gnutls_get_asn_mpis(ASN1_TYPE asn, const char *root,
+int _gnutls_get_asn_mpis(asn1_node asn, const char *root,
 			 gnutls_pk_params_st * params);
 
 int _gnutls_get_key_id(gnutls_pk_params_st *,
@@ -229,13 +235,13 @@ void _asnstr_append_name(char *name, size_t name_size, const char *part1,
 
 /* Given a @c2 which it returns an allocated DER encoding of @whom in @out */
 inline static int
-_gnutls_x509_get_raw_field(ASN1_TYPE c2, const char *whom, gnutls_datum_t *out)
+_gnutls_x509_get_raw_field(asn1_node c2, const char *whom, gnutls_datum_t *out)
 {
 	return _gnutls_x509_der_encode(c2, whom, out, 0);
 }
 
 int
-_gnutls_x509_get_raw_field2(ASN1_TYPE c2, const gnutls_datum_t * raw,
+_gnutls_x509_get_raw_field2(asn1_node c2, const gnutls_datum_t * raw,
 			 const char *whom, gnutls_datum_t * dn);
 
 unsigned
@@ -256,11 +262,11 @@ unsigned _gnutls_check_key_purpose(gnutls_x509_crt_t cert, const char *purpose,
 
 time_t _gnutls_x509_generalTime2gtime(const char *ttime);
 
-int _gnutls_get_extension(ASN1_TYPE asn, const char *root,
+int _gnutls_get_extension(asn1_node asn, const char *root,
 		  const char *extension_id, int indx,
 		  gnutls_datum_t * ret, unsigned int *_critical);
 
-int _gnutls_set_extension(ASN1_TYPE asn, const char *root,
+int _gnutls_set_extension(asn1_node asn, const char *root,
 		  const char *ext_id,
 		  const gnutls_datum_t * ext_data, unsigned int critical);
 
@@ -276,7 +282,7 @@ int _gnutls_x509_decode_ext(const gnutls_datum_t *der, gnutls_x509_ext_st *out);
 int _gnutls_x509_raw_crt_to_raw_pubkey(const gnutls_datum_t * cert,
 			   gnutls_datum_t * rpubkey);
 
-int _gnutls_x509_get_version(ASN1_TYPE root, const char *name);
+int _gnutls_x509_get_version(asn1_node root, const char *name);
 
 int x509_crt_to_raw_pubkey(gnutls_x509_crt_t crt,
 			   gnutls_datum_t * rpubkey);
