@@ -96,7 +96,8 @@ static int validate_name_constraints_node(gnutls_x509_subject_alt_name_t type,
 {
 	if (type != GNUTLS_SAN_DNSNAME && type != GNUTLS_SAN_RFC822NAME &&
 		type != GNUTLS_SAN_DN && type != GNUTLS_SAN_URI &&
-		type != GNUTLS_SAN_IPADDRESS) {
+		type != GNUTLS_SAN_IPADDRESS && 
+		type != GNUTLS_SAN_OTHERNAME_MSUSERPRINCIPAL) {
 		return gnutls_assert_val(GNUTLS_E_X509_UNKNOWN_SAN);
 	}
 
@@ -138,6 +139,28 @@ int _gnutls_extract_name_constraints(asn1_node c2, const char *vstr,
 			break;
 		}
 
+		if (type == GNUTLS_SAN_OTHERNAME) {
+			gnutls_datum_t oid = { NULL, 0 };
+                        gnutls_datum_t parsed_othername = { NULL, 0 };
+                        ret = _gnutls_parse_general_name2(c2, tmpstr, -1, &oid, &type, 1);
+                        if(ret < 0) {
+                                gnutls_assert();
+                                goto cleanup;
+                        }
+
+                        ret = gnutls_x509_othername_to_virtual((char*)oid.data, &tmp, &type,
+                                &parsed_othername);
+                        if(ret < 0) {
+                                gnutls_assert();
+                                goto cleanup;
+                        }
+
+			gnutls_free(oid.data);
+			gnutls_free(tmp.data);
+
+                        memcpy(&tmp, &parsed_othername, sizeof(gnutls_datum_t));
+		}
+
 		ret = validate_name_constraints_node(type, &tmp);
 		if (ret < 0) {
 			gnutls_assert();
