@@ -820,6 +820,14 @@ gnutls_session_t init_tls_session(const char *host)
 		}
 	}
 
+	if (HAVE_OPT(COMPRESS_CERT) && disable_extensions == 0) {
+		ret = compress_cert_set_methods(session,
+						OPTS_ARRAY(COMPRESS_CERT),
+						OPTS_COUNT(COMPRESS_CERT));
+		if (ret < 0)
+			exit(1);
+	}
+
 	if (HAVE_OPT(HEARTBEAT))
 		gnutls_heartbeat_enable(session,
 					GNUTLS_HB_PEER_ALLOWED_TO_SEND);
@@ -1646,6 +1654,7 @@ static void cmd_parser(int argc, char **argv)
 		for (p = gnutls_get_library_config(); p->name; p++) {
 			log_msg(stdout, "%s: %s\n", p->name, p->value);
 		}
+		log_msg(stdout, "system-config: %s\n", gnutls_get_system_config_file());
 		exit(0);
 	}
 
