@@ -291,6 +291,53 @@ int cert_verify(gnutls_session_t session, const char *hostname, const char *purp
 	return 1;
 }
 
+/* Parse input string and set certificate compression methods */
+int compress_cert_set_methods(gnutls_session_t session,
+			      const char **strings,
+			      size_t n_strings)
+{
+	int ret = 0;
+	gnutls_compression_method_t *methods;
+
+	if (n_strings == 0) {
+		return 0;
+	}
+
+/* GCC analyzer in 11.2 mishandles reallocarray/free */
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wanalyzer-mismatching-deallocation"
+
+	methods = reallocarray(NULL, n_strings, sizeof(*methods));
+	if (!methods) {
+		fprintf(stderr, "Could not set certificate compression methods: %s\n",
+			gnutls_strerror(ret));
+		return GNUTLS_E_MEMORY_ERROR;
+	}
+
+	for (size_t i = 0; i < n_strings; ++i) {
+		methods[i] = gnutls_compression_get_id(strings[i]);
+		if (methods[i] == GNUTLS_COMP_UNKNOWN) {
+			fprintf(stderr, "Unknown compression method: %s\n",
+				strings[i]);
+			goto cleanup;
+		}
+	}
+
+	ret = gnutls_compress_certificate_set_methods(session, methods, n_strings);
+	if (ret < 0) {
+		fprintf(stderr, "Could not set certificate compression methods: %s\n",
+			gnutls_strerror(ret));
+		goto cleanup;
+	}
+
+cleanup:
+	free(methods);
+
+#pragma GCC diagnostic pop
+
+	return ret;
+}
+
 static void
 print_dh_info(gnutls_session_t session, const char *str, int print)
 {
@@ -395,6 +442,51 @@ static void print_ecdh_info(gnutls_session_t session, const char *str, int print
 
 }
 
+struct channel_binding_request {
+	gnutls_channel_binding_t type;
+	const char *name;
+};
+
+static void print_channel_bindings(gnutls_session_t session, int print)
+{
+	static const struct channel_binding_request requests[] = {
+		{ GNUTLS_CB_TLS_UNIQUE, "tls-unique" },
+		{ GNUTLS_CB_TLS_SERVER_END_POINT, "tls-server-end-point" },
+		{ GNUTLS_CB_TLS_EXPORTER, "tls-exporter" }
+	};
+	size_t i;
+
+	if (!print) {
+		return;
+	}
+
+	log_msg(stdout, "- Channel bindings\n");
+	for (i = 0; i < sizeof(requests) / sizeof(requests[0]); i++) {
+		gnutls_datum_t cb;
+		int rc;
+
+		rc = gnutls_session_channel_binding(session,
+						    requests[i].type,
+						    &cb);
+		if (rc == GNUTLS_E_CHANNEL_BINDING_NOT_AVAILABLE) {
+			log_msg(stdout, " - '%s': not available\n",
+				requests[i].name);
+		} else if (rc < 0) {
+			fprintf(stderr, " - '%s': error: %s\n",
+				requests[i].name, gnutls_strerror(rc));
+		} else {
+			size_t j;
+
+			log_msg(stdout, " - '%s': ",
+				requests[i].name);
+			for (j = 0; j < cb.size; j++)
+				log_msg(stdout, "%02x", cb.data[j]);
+			log_msg(stdout, "\n");
+			gnutls_free(cb.data);
+		}
+	}
+}
+
 int print_info(gnutls_session_t session, int verbose, int flags)
 {
 	const char *tmp;
@@ -552,25 +644,7 @@ int print_info(gnutls_session_t session, int verbose, int flags)
 		log_msg(stdout, "- Application protocol: %.*s\n", p.size, p.data);
 #endif
 
-	if (verbose) {
-		gnutls_datum_t cb;
-
-		rc = gnutls_session_channel_binding(session,
-						    GNUTLS_CB_TLS_UNIQUE,
-						    &cb);
-		if (rc)
-			fprintf(stderr, "Channel binding error: %s\n",
-				gnutls_strerror(rc));
-		else {
-			size_t i;
-
-			log_msg(stdout, "- Channel binding 'tls-unique': ");
-			for (i = 0; i < cb.size; i++)
-				log_msg(stdout, "%02x", cb.data[i]);
-			log_msg(stdout, "\n");
-			gnutls_free(cb.data);
-		}
-	}
+	print_channel_bindings(session, verbose);
 
 	fflush(stdout);
 
