@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see
- * <http://www.gnu.org/licenses/>.
+ * <https://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -37,11 +37,13 @@ int main(int argc, char **argv)
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#include <errno.h>
 #include <gnutls/gnutls.h>
-#include <psktool-args.h>
+#include "psktool-options.h"
 
 #include <gnutls/crypto.h>	/* for random */
 
+#include <stdbool.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 
@@ -54,13 +56,15 @@ int main(int argc, char **argv)
 
 /* Gnulib portability files. */
 #include <minmax.h>
+#include "close-stream.h"
 #include "getpass.h"
+#include "xsize.h"
 
-static int write_key(const char *username, const char *key, int key_size,
+static int write_key(const char *username,
+		     const unsigned char *key, size_t key_size,
 		     const char *passwd_file);
 
-#define KPASSWD "/etc/passwd.psk"
-#define MAX_KEY_SIZE 64
+#define MAX_KEY_SIZE 512
 int main(int argc, char **argv)
 {
 	int ret;
@@ -68,11 +72,8 @@ int main(int argc, char **argv)
 	struct passwd *pwd;
 #endif
 	unsigned char key[MAX_KEY_SIZE];
-	char hex_key[MAX_KEY_SIZE * 2 + 1];
-	int optct, key_size;
-	gnutls_datum_t dkey;
+	size_t key_size;
 	const char *passwd, *username;
-	size_t hex_key_size = sizeof(hex_key);
 
 	if ((ret = gnutls_global_init()) < 0) {
 		fprintf(stderr, "global_init: %s\n", gnutls_strerror(ret));
@@ -81,14 +82,13 @@ int main(int argc, char **argv)
 
 	umask(066);
 
-	optct = optionProcess(&psktoolOptions, argc, argv);
-	argc -= optct;
-	argv += optct;
+	optionProcess(&psktoolOptions, argc, argv);
 
-	if (!HAVE_OPT(PASSWD))
-		passwd = (char *) KPASSWD;
-	else
-		passwd = OPT_ARG(PASSWD);
+	if (!HAVE_OPT(PSKFILE)) {
+		fprintf(stderr, "You need to specify a PSK key file\n");
+		exit(1);
+	} else
+		passwd = OPT_ARG(PSKFILE);
 
 	if (!HAVE_OPT(USERNAME)) {
 #ifndef _WIN32
@@ -113,7 +113,7 @@ int main(int argc, char **argv)
 	}
 
 	if (!HAVE_OPT(KEYSIZE) || OPT_VALUE_KEYSIZE < 1)
-		key_size = 16;
+		key_size = 32;
 	else
 		key_size = OPT_VALUE_KEYSIZE;
 
@@ -125,16 +125,7 @@ int main(int argc, char **argv)
 		exit(1);
 	}
 
-	dkey.data = key;
-	dkey.size = key_size;
-
-	ret = gnutls_hex_encode(&dkey, hex_key, &hex_key_size);
-	if (ret < 0) {
-		fprintf(stderr, "HEX encoding error\n");
-		exit(1);
-	}
-
-	ret = write_key(username, hex_key, hex_key_size, passwd);
+	ret = write_key(username, key, key_size, passwd);
 	if (ret == 0)
 		printf("Key stored to %s\n", passwd);
 
@@ -143,53 +134,52 @@ int main(int argc, char **argv)
 
 static int filecopy(const char *src, const char *dst)
 {
-	FILE *fd, *fd2;
+	FILE *fp, *fp2;
 	char line[5 * 1024];
 	char *p;
 
-	fd = fopen(dst, "w");
-	if (fd == NULL) {
+	fp = fopen(dst, "w");
+	if (fp == NULL) {
 		fprintf(stderr, "Cannot open '%s' for write\n", dst);
 		return -1;
 	}
 
-	fd2 = fopen(src, "r");
-	if (fd2 == NULL) {
+	fp2 = fopen(src, "r");
+	if (fp2 == NULL) {
 		/* empty file */
-		fclose(fd);
+		fclose(fp);
 		return 0;
 	}
 
 	line[sizeof(line) - 1] = 0;
 	do {
-		p = fgets(line, sizeof(line) - 1, fd2);
+		p = fgets(line, sizeof(line) - 1, fp2);
 		if (p == NULL)
 			break;
 
-		fputs(line, fd);
+		fputs(line, fp);
 	}
 	while (1);
 
-	fclose(fd);
-	fclose(fd2);
+	fclose(fp);
+	fclose(fp2);
 
 	return 0;
 }
 
 static int
-write_key(const char *username, const char *key, int key_size,
+write_key(const char *username, const unsigned char *key, size_t key_size,
 	  const char *passwd_file)
 {
-	FILE *fd;
+	FILE *fp;
 	char line[5 * 1024];
 	char *p, *pp;
 	char tmpname[1024];
-
-
-	/* delete previous entry */
+	gnutls_datum_t tmp, _username = { NULL, 0 }, _key = { NULL, 0 };
 	struct stat st;
-	FILE *fd2;
-	int put;
+	FILE *fp2;
+	bool put;
+	int ret = 0;
 
 	if (strlen(passwd_file) + 5 > sizeof(tmpname)) {
 		fprintf(stderr, "file '%s' is tooooo long\n", passwd_file);
@@ -209,25 +199,76 @@ write_key(const char *username, const char *key, int key_size,
 		return -1;
 	}
 
-	fd = fopen(passwd_file, "w");
-	if (fd == NULL) {
+	fp = fopen(passwd_file, "w");
+	if (fp == NULL) {
 		fprintf(stderr, "Cannot open '%s' for write\n",
 			passwd_file);
-		remove(tmpname);
+		(void)remove(tmpname);
 		return -1;
 	}
 
-	fd2 = fopen(tmpname, "r");
-	if (fd2 == NULL) {
+	fp2 = fopen(tmpname, "r");
+	if (fp2 == NULL) {
 		fprintf(stderr, "Cannot open '%s' for read\n", tmpname);
-		remove(tmpname);
-		fclose(fd);
+		(void)remove(tmpname);
+		fclose(fp);
 		return -1;
 	}
 
-	put = 0;
-	do {
-		p = fgets(line, sizeof(line) - 1, fd2);
+	/* encode username if it contains special characters */
+	if (strcspn(username, ":\n") != strlen(username)) {
+		char *new_data;
+		size_t new_size;
+
+		tmp.data = (void *)username;
+		tmp.size = strlen(username);
+
+		ret = gnutls_hex_encode2(&tmp, &_username);
+		if (ret < 0) {
+			fprintf(stderr, "HEX encoding error\n");
+			ret = -1;
+			goto out;
+		}
+
+		/* prepend '#' */
+		new_size = xsum(_username.size, 2);
+		if (size_overflow_p(new_size)) {
+			ret = -1;
+			goto out;
+		}
+		new_data = gnutls_realloc(_username.data, new_size);
+		if (!new_data) {
+			ret = -1;
+			goto out;
+		}
+		memmove(new_data + 1, new_data, _username.size);
+		new_data[0] = '#';
+		new_data[_username.size + 1] = '\0';
+		_username.data = (void *)new_data;
+		_username.size = new_size - 1;
+	} else {
+		_username.data = (void *)gnutls_strdup(username);
+		if (!_username.data) {
+			ret = -1;
+			goto out;
+		}
+		_username.size = strlen(username);
+	}
+
+	/* encode key */
+	tmp.data = (void *)key;
+	tmp.size = key_size;
+
+	ret = gnutls_hex_encode2(&tmp, &_key);
+	if (ret < 0) {
+		fprintf(stderr, "HEX encoding error\n");
+		ret = -1;
+		goto out;
+	}
+
+	put = false;
+	while (true) {
+		p = fgets(line, sizeof(line) - 1, fp2);
 		if (p == NULL)
 			break;
 
@@ -235,28 +276,34 @@ write_key(const char *username, const char *key, int key_size,
 		if (pp == NULL)
 			continue;
 
-		if (strncmp(p, username,
-			    MAX(strlen(username),
+		if (strncmp(p, (const char *) _username.data,
+			    MAX(_username.size,
 				(unsigned int) (pp - p))) == 0) {
-			put = 1;
-			fprintf(fd, "%s:%s\n", username, key);
+			put = true;
+			fprintf(fp, "%s:%s\n", _username.data, _key.data);
 		} else {
-			fputs(line, fd);
+			fputs(line, fp);
 		}
 	}
-	while (1);
 
-	if (put == 0) {
-		fprintf(fd, "%s:%s\n", username, key);
+	if (!put) {
+		fprintf(fp, "%s:%s\n", _username.data, _key.data);
 	}
 
-	fclose(fd);
-	fclose(fd2);
+ out:
+	if (close_stream(fp) == EOF) {
+		fprintf(stderr, "Error writing %s: %s\n",
+			passwd_file, strerror(errno));
+		ret = -1;
+	}
 
-	remove(tmpname);
+	fclose(fp2);
 
+	(void)remove(tmpname);
+	gnutls_free(_username.data);
+	gnutls_free(_key.data);
 
-	return 0;
+	return ret;
 }
 
 #endif				/* ENABLE_PSK */
