@@ -587,17 +587,26 @@ test_code_t test_dhe_group(gnutls_session_t session)
 			gnutls_datum_t p3;
 			
 			ret2 = gnutls_dh_params_init(&dhp);
-			if (ret2 < 0)
+			if (ret2 < 0) {
+				fclose(fp);
 				return TEST_FAILED;
+			}
 
 			ret2 = gnutls_dh_params_import_raw(dhp, &prime, &gen);
-			if (ret2 < 0)
+			if (ret2 < 0) {
+				gnutls_dh_params_deinit(dhp);
+				fclose(fp);
 				return TEST_FAILED;
+			}
 
 			ret2 = gnutls_dh_params_export2_pkcs3(dhp, GNUTLS_X509_FMT_PEM, &p3);
-			if (ret2 < 0)
+			if (ret2 < 0) {
+				gnutls_dh_params_deinit(dhp);
+				fclose(fp);
 				return TEST_FAILED;
+			}
 
+			gnutls_dh_params_deinit(dhp);
 			fprintf(fp, "\n%s\n", p3.data);
 			gnutls_free(p3.data);
 		}
@@ -1582,6 +1591,7 @@ test_code_t test_chain_order(gnutls_session_t session)
 	p_size = 0;
 	pos = NULL;
 	for (i=0;i<cert_list_size;i++) {
+		char *new_p;
 		t.data = NULL;
 		ret = gnutls_pem_base64_encode_alloc("CERTIFICATE", &cert_list[i], &t);
 		if (ret < 0) {
@@ -1589,7 +1599,12 @@ test_code_t test_chain_order(gnutls_session_t session)
 			return TEST_FAILED;
 		}
 
-		p = realloc(p, p_size+t.size+1);
+		new_p = realloc(p, p_size+t.size+1);
+		if (!new_p) {
+			free(p);
+			return TEST_FAILED;
+		}
+		p = new_p;
 		pos = p + p_size;
 
 		memcpy(pos, t.data, t.size);
