@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see
- * <http://www.gnu.org/licenses/>.
+ * <https://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -44,23 +44,39 @@
 #include <read-file.h>
 
 #include "certtool-common.h"
-#include "tpmtool-args.h"
+#include "tpmtool-options.h"
+#include "common.h"
 
 static void cmd_parser(int argc, char **argv);
 static void tpm_generate(FILE * outfile, unsigned int key_type,
-			 unsigned int bits, unsigned int flags);
-static void tpm_pubkey(const char *url, FILE * outfile);
-static void tpm_delete(const char *url, FILE * outfile);
+			 unsigned int bits, unsigned int flags,
+			 unsigned int srk_well_known);
+static void tpm_pubkey(const char *url, FILE * outfile,
+		       unsigned int srk_well_known);
+static void tpm_delete(const char *url, FILE * outfile,
+		       unsigned int srk_well_known);
+static void tpm_test_sign(const char *url, FILE * outfile);
 static void tpm_list(FILE * outfile);
 
 static gnutls_x509_crt_fmt_t incert_format, outcert_format;
 static gnutls_tpmkey_fmt_t inkey_format, outkey_format;
 
 static FILE *outfile;
+static const char *outfile_name = NULL;
 static FILE *infile;
 int batch = 0;
 int ask_pass = 0;
 
+void app_exit(int val)
+{
+	if (val != 0) {
+		if (outfile_name != NULL) {
+			remove(outfile_name);
+		}
+	}
+	exit(val);
+}
+
 static void tls_log_func(int level, const char *str)
 {
 	fprintf(stderr, "|<%d>| %s", level, str);
@@ -77,7 +93,6 @@ int main(int argc, char **argv)
 static void cmd_parser(int argc, char **argv)
 {
 	int ret, debug = 0;
-	unsigned int optct;
 	unsigned int key_type = GNUTLS_PK_UNKNOWN;
 	unsigned int bits = 0;
 	unsigned int genflags = 0;
@@ -86,9 +101,7 @@ static void cmd_parser(int argc, char **argv)
 	 */
 	const char *sec_param = "legacy";
 
-	optct = optionProcess(&tpmtoolOptions, argc, argv);
-	argc += optct;
-	argv += optct;
+	optionProcess(&tpmtoolOptions, argc, argv);
 
 	if (HAVE_OPT(DEBUG))
 		debug = OPT_VALUE_DEBUG;
@@ -122,23 +135,24 @@ static void cmd_parser(int argc, char **argv)
 		printf("Setting log level to %d\n", debug);
 
 	if ((ret = gnutls_global_init()) < 0) {
-		fprintf(stderr, "global_init: %s", gnutls_strerror(ret));
+		fprintf(stderr, "global_init: %s\n", gnutls_strerror(ret));
 		exit(1);
 	}
 
 	if (HAVE_OPT(OUTFILE)) {
 		outfile = safe_open_rw(OPT_ARG(OUTFILE), 0);
 		if (outfile == NULL) {
-			fprintf(stderr, "%s", OPT_ARG(OUTFILE));
+			fprintf(stderr, "%s\n", OPT_ARG(OUTFILE));
 			exit(1);
 		}
+		outfile_name = OPT_ARG(OUTFILE);
 	} else
 		outfile = stdout;
 
 	if (HAVE_OPT(INFILE)) {
 		infile = fopen(OPT_ARG(INFILE), "rb");
 		if (infile == NULL) {
-			fprintf(stderr, "%s", OPT_ARG(INFILE));
+			fprintf(stderr, "%s\n", OPT_ARG(INFILE));
 			exit(1);
 		}
 	} else
@@ -153,13 +167,15 @@ static void cmd_parser(int argc, char **argv)
 	if (HAVE_OPT(GENERATE_RSA)) {
 		key_type = GNUTLS_PK_RSA;
 		bits = get_bits(key_type, bits, sec_param, 0);
-		tpm_generate(outfile, key_type, bits, genflags);
+		tpm_generate(outfile, key_type, bits, genflags, HAVE_OPT(SRK_WELL_KNOWN));
 	} else if (HAVE_OPT(PUBKEY)) {
-		tpm_pubkey(OPT_ARG(PUBKEY), outfile);
+		tpm_pubkey(OPT_ARG(PUBKEY), outfile, HAVE_OPT(SRK_WELL_KNOWN));
 	} else if (HAVE_OPT(DELETE)) {
-		tpm_delete(OPT_ARG(DELETE), outfile);
+		tpm_delete(OPT_ARG(DELETE), outfile, HAVE_OPT(SRK_WELL_KNOWN));
 	} else if (HAVE_OPT(LIST)) {
 		tpm_list(outfile);
+	} else if (HAVE_OPT(TEST_SIGN)) {
+		tpm_test_sign(OPT_ARG(TEST_SIGN), outfile);
 	} else {
 		USAGE(1);
 	}
@@ -169,21 +185,93 @@ static void cmd_parser(int argc, char **argv)
 	gnutls_global_deinit();
 }
 
-static void tpm_generate(FILE * outfile, unsigned int key_type,
-			 unsigned int bits, unsigned int flags)
+#define TEST_DATA "Test data to sign"
+
+static void
+tpm_test_sign(const char *url, FILE * out)
+{
+	gnutls_privkey_t privkey;
+	gnutls_pubkey_t pubkey;
+	int ret;
+	gnutls_datum_t data, sig = {NULL, 0};
+	int pk;
+
+	pkcs11_common(NULL);
+
+	data.data = (void*)TEST_DATA;
+	data.size = sizeof(TEST_DATA)-1;
+
+	ret = gnutls_privkey_init(&privkey);
+	if (ret < 0) {
+		fprintf(stderr, "Error in %s:%d: %s\n", __func__,
+			__LINE__, gnutls_strerror(ret));
+		exit(1);
+	}
+
+	ret = gnutls_pubkey_init(&pubkey);
+	if (ret < 0) {
+		fprintf(stderr, "Error in %s:%d: %s\n", __func__,
+			__LINE__, gnutls_strerror(ret));
+		exit(1);
+	}
+
+	ret = gnutls_privkey_import_url(privkey, url, 0);
+	if (ret < 0) {
+		fprintf(stderr, "Cannot import private key: %s\n",
+			gnutls_strerror(ret));
+		exit(1);
+	}
+
+	ret = gnutls_pubkey_import_tpm_url(pubkey, url, NULL, 0);
+	if (ret < 0) {
+		fprintf(stderr, "Cannot import public key: %s\n",
+			gnutls_strerror(ret));
+		exit(1);
+	}
+
+	ret = gnutls_privkey_sign_data(privkey, GNUTLS_DIG_SHA1, 0, &data, &sig);
+	if (ret < 0) {
+		fprintf(stderr, "Cannot sign data: %s\n",
+			gnutls_strerror(ret));
+		exit(1);
+	}
+
+	pk = gnutls_pubkey_get_pk_algorithm(pubkey, NULL);
+
+	fprintf(stderr, "Verifying against private key parameters... ");
+	ret = gnutls_pubkey_verify_data2(pubkey, gnutls_pk_to_sign(pk, GNUTLS_DIG_SHA1),
+		0, &data, &sig);
+	if (ret < 0) {
+		fprintf(stderr, "Cannot verify signed data: %s\n",
+			gnutls_strerror(ret));
+		exit(1);
+	}
+
+	fprintf(stderr, "ok\n");
+
+	gnutls_free(sig.data);
+	gnutls_pubkey_deinit(pubkey);
+	gnutls_privkey_deinit(privkey);
+}
+
+static void tpm_generate(FILE * out, unsigned int key_type,
+			 unsigned int bits, unsigned int flags,
+			 unsigned int srk_well_known)
 {
 	int ret;
-	char *srk_pass, *key_pass = NULL;
+	char *srk_pass = NULL, *key_pass = NULL;
 	gnutls_datum_t privkey, pubkey;
 
-	srk_pass = getpass("Enter SRK password: ");
-	if (srk_pass != NULL)
-		srk_pass = strdup(srk_pass);
+	if (!srk_well_known) {
+		srk_pass = getpass("Enter SRK password: ");
+		if (srk_pass != NULL)
+			srk_pass = strdup(srk_pass);
+	}
 
 	if (!(flags & GNUTLS_TPM_REGISTER_KEY)) {
 		key_pass = getpass("Enter key password: ");
 		if (key_pass != NULL)
-			key_pass = strdup(srk_pass);
+			key_pass = strdup(key_pass);
 	}
 
 	ret =
@@ -195,38 +283,39 @@ static void tpm_generate(FILE * outfile, unsigned int key_type,
 	free(srk_pass);
 
 	if (ret < 0) {
-		fprintf(stderr, "gnutls_tpm_privkey_generate: %s",
+		fprintf(stderr, "gnutls_tpm_privkey_generate: %s\n",
 			gnutls_strerror(ret));
 		exit(1);
 	}
 
-/*  fwrite (pubkey.data, 1, pubkey.size, outfile);
-  fputs ("\n", outfile);*/
-	fwrite(privkey.data, 1, privkey.size, outfile);
-	fputs("\n", outfile);
+
+	fwrite(privkey.data, 1, privkey.size, out);
+	fputs("\n", out);
 
 	gnutls_free(privkey.data);
 	gnutls_free(pubkey.data);
 }
 
-static void tpm_delete(const char *url, FILE * outfile)
+static void tpm_delete(const char *url, FILE * out,
+		       unsigned int srk_well_known)
 {
 	int ret;
-	char *srk_pass;
+	char *srk_pass = NULL;
 
-	srk_pass = getpass("Enter SRK password: ");
+	if (!srk_well_known)
+		srk_pass = getpass("Enter SRK password: ");
 
 	ret = gnutls_tpm_privkey_delete(url, srk_pass);
 	if (ret < 0) {
-		fprintf(stderr, "gnutls_tpm_privkey_delete: %s",
+		fprintf(stderr, "gnutls_tpm_privkey_delete: %s\n",
 			gnutls_strerror(ret));
 		exit(1);
 	}
 
-	fprintf(outfile, "Key %s deleted\n", url);
+	fprintf(out, "Key %s deleted\n", url);
 }
 
-static void tpm_list(FILE * outfile)
+static void tpm_list(FILE * out)
 {
 	int ret;
 	gnutls_tpm_key_list_t list;
@@ -235,38 +324,42 @@ static void tpm_list(FILE * outfile)
 
 	ret = gnutls_tpm_get_registered(&list);
 	if (ret < 0) {
-		fprintf(stderr, "gnutls_tpm_get_registered: %s",
+		fprintf(stderr, "gnutls_tpm_get_registered: %s\n",
 			gnutls_strerror(ret));
 		exit(1);
 	}
 
-	fprintf(outfile, "Available keys:\n");
+	fprintf(out, "Available keys:\n");
 	for (i = 0;; i++) {
 		ret = gnutls_tpm_key_list_get_url(list, i, &url, 0);
 		if (ret == GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE)
 			break;
 		else if (ret < 0) {
-			fprintf(stderr, "gnutls_tpm_key_list_get_url: %s",
+			fprintf(stderr, "gnutls_tpm_key_list_get_url: %s\n",
 				gnutls_strerror(ret));
+			gnutls_tpm_key_list_deinit(list);
 			exit(1);
 		}
 
-		fprintf(outfile, "\t%u: %s\n", i, url);
+		fprintf(out, "\t%u: %s\n", i, url);
 		gnutls_free(url);
 	}
 
-	fputs("\n", outfile);
+	gnutls_tpm_key_list_deinit(list);
+	fputs("\n", out);
 }
 
-static void tpm_pubkey(const char *url, FILE * outfile)
+static void tpm_pubkey(const char *url, FILE * out, unsigned int srk_well_known)
 {
 	int ret;
-	char *srk_pass;
+	char *srk_pass = NULL;
 	gnutls_pubkey_t pubkey;
 
-	srk_pass = getpass("Enter SRK password: ");
-	if (srk_pass != NULL)
-		srk_pass = strdup(srk_pass);
+	if (!srk_well_known) {
+		srk_pass = getpass("Enter SRK password: ");
+		if (srk_pass != NULL)
+			srk_pass = strdup(srk_pass);
+	}
 
 	gnutls_pubkey_init(&pubkey);
 
@@ -275,12 +368,12 @@ static void tpm_pubkey(const char *url, FILE * outfile)
 	free(srk_pass);
 
 	if (ret < 0) {
-		fprintf(stderr, "gnutls_pubkey_import_tpm_url: %s",
+		fprintf(stderr, "gnutls_pubkey_import_tpm_url: %s\n",
 			gnutls_strerror(ret));
 		exit(1);
 	}
 
-	_pubkey_info(outfile, GNUTLS_CRT_PRINT_FULL, pubkey);
+	print_pubkey_info(pubkey, out, GNUTLS_CRT_PRINT_FULL, GNUTLS_X509_FMT_PEM, 1);
 
 	gnutls_pubkey_deinit(pubkey);
 }
