@@ -16,31 +16,54 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with GnuTLS.  If not, see <https://www.gnu.org/licenses/>.
+ * along with GnuTLS; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 #include <assert.h>
 #include <stdio.h>
-#include "utils.h"
+#include <utils.h>
 #include <gnutls/gnutls.h>
 #include <gnutls/abstract.h>
 #include <gnutls/x509.h>
 
-void generate_successfully(gnutls_privkey_t *privkey, gnutls_pubkey_t *pubkey,
-			   unsigned int size);
-void generate_unsuccessfully(gnutls_privkey_t *privkey, gnutls_pubkey_t *pubkey,
-			     unsigned int size);
+#define FIPS_PUSH_CONTEXT() do {				\
+	ret = gnutls_fips140_push_context(fips_context);	\
+	if (ret < 0) {						\
+		fail("gnutls_fips140_push_context failed\n");	\
+	}							\
+} while (0)
+
+#define FIPS_POP_CONTEXT(state) do {					\
+	ret = gnutls_fips140_pop_context();				\
+	if (ret < 0) {							\
+		fail("gnutls_fips140_context_pop failed\n");		\
+	}								\
+	fips_state = gnutls_fips140_get_operation_state(fips_context);	\
+	if (fips_state != GNUTLS_FIPS140_OP_ ## state) {		\
+		fail("operation state is not " # state " (%d)\n",	\
+		     fips_state);					\
+	}								\
+} while (0)
+
+
+void generate_successfully(gnutls_privkey_t* privkey, gnutls_pubkey_t* pubkey,
+                           unsigned int size);
+void generate_unsuccessfully(gnutls_privkey_t* privkey, gnutls_pubkey_t* pubkey,
+                             unsigned int size);
 void sign_verify_successfully(gnutls_privkey_t privkey, gnutls_pubkey_t pubkey);
 void sign_verify_unsuccessfully(gnutls_privkey_t privkey,
-				gnutls_pubkey_t pubkey);
+                                gnutls_pubkey_t pubkey);
 void nosign_verify(gnutls_privkey_t privkey, gnutls_pubkey_t pubkey);
 
-void generate_successfully(gnutls_privkey_t *privkey, gnutls_pubkey_t *pubkey,
-			   unsigned int size)
+
+void generate_successfully(gnutls_privkey_t* privkey, gnutls_pubkey_t* pubkey,
+                           unsigned int size)
 {
 	int ret;
 	gnutls_x509_privkey_t xprivkey;
 	gnutls_fips140_context_t fips_context;
+	gnutls_fips140_operation_state_t fips_state;
 	assert(gnutls_fips140_context_init(&fips_context) == 0);
 
 	fprintf(stderr, "%d-bit\n", size);
@@ -72,12 +95,14 @@ void generate_successfully(gnutls_privkey_t *privkey, gnutls_pubkey_t *pubkey,
 	gnutls_fips140_context_deinit(fips_context);
 }
 
-void generate_unsuccessfully(gnutls_privkey_t *privkey, gnutls_pubkey_t *pubkey,
-			     unsigned int size)
+
+void generate_unsuccessfully(gnutls_privkey_t* privkey, gnutls_pubkey_t* pubkey,
+                             unsigned int size)
 {
 	int ret;
 	gnutls_x509_privkey_t xprivkey;
 	gnutls_fips140_context_t fips_context;
+	gnutls_fips140_operation_state_t fips_state;
 	assert(gnutls_fips140_context_init(&fips_context) == 0);
 
 	fprintf(stderr, "%d-bit\n", size);
@@ -127,21 +152,23 @@ void generate_unsuccessfully(gnutls_privkey_t *privkey, gnutls_pubkey_t *pubkey,
 	gnutls_fips140_context_deinit(fips_context);
 }
 
-void sign_verify_successfully(gnutls_privkey_t privkey, gnutls_pubkey_t pubkey)
-{
+
+void sign_verify_successfully(gnutls_privkey_t privkey, gnutls_pubkey_t pubkey) {
 	int ret;
 	gnutls_fips140_context_t fips_context;
+	gnutls_fips140_operation_state_t fips_state;
 
 	gnutls_datum_t signature;
 	gnutls_datum_t plaintext = {
-		.data = (unsigned char *const)"Hello world!", .size = 12
+		.data = (unsigned char* const) "Hello world!",
+		.size = 12
 	};
 	assert(gnutls_fips140_context_init(&fips_context) == 0);
 
 	/* RSA sign: approved */
 	FIPS_PUSH_CONTEXT();
 	ret = gnutls_privkey_sign_data(privkey, GNUTLS_DIG_SHA256, 0,
-				       &plaintext, &signature);
+	                               &plaintext, &signature);
 	if (ret < 0)
 		fail("gnutls_privkey_sign_data failed\n");
 	FIPS_POP_CONTEXT(APPROVED);
@@ -149,7 +176,7 @@ void sign_verify_successfully(gnutls_privkey_t privkey, gnutls_pubkey_t pubkey)
 	/* RSA verify: approved */
 	FIPS_PUSH_CONTEXT();
 	ret = gnutls_pubkey_verify_data2(pubkey, GNUTLS_SIGN_RSA_SHA256, 0,
-					 &plaintext, &signature);
+	                                 &plaintext, &signature);
 	if (ret < 0)
 		fail("gnutls_pubkey_verify_data2 failed\n");
 	FIPS_POP_CONTEXT(APPROVED);
@@ -158,22 +185,24 @@ void sign_verify_successfully(gnutls_privkey_t privkey, gnutls_pubkey_t pubkey)
 	gnutls_fips140_context_deinit(fips_context);
 }
 
+
 void sign_verify_unsuccessfully(gnutls_privkey_t privkey,
-				gnutls_pubkey_t pubkey)
-{
+                                gnutls_pubkey_t pubkey) {
 	int ret;
 	gnutls_fips140_context_t fips_context;
+	gnutls_fips140_operation_state_t fips_state;
 
 	gnutls_datum_t signature;
 	gnutls_datum_t plaintext = {
-		.data = (unsigned char *const)"Hello world!", .size = 12
+		.data = (unsigned char* const) "Hello world!",
+		.size = 12
 	};
 	assert(gnutls_fips140_context_init(&fips_context) == 0);
 
 	/* small key RSA sign: not approved */
 	FIPS_PUSH_CONTEXT();
 	ret = gnutls_privkey_sign_data(privkey, GNUTLS_DIG_SHA256, 0,
-				       &plaintext, &signature);
+	                               &plaintext, &signature);
 	if (ret < 0)
 		fail("gnutls_privkey_sign_data failed\n");
 	FIPS_POP_CONTEXT(NOT_APPROVED);
@@ -181,7 +210,7 @@ void sign_verify_unsuccessfully(gnutls_privkey_t privkey,
 	/* small key RSA verify: not approved */
 	FIPS_PUSH_CONTEXT();
 	ret = gnutls_pubkey_verify_data2(pubkey, GNUTLS_SIGN_RSA_SHA256, 0,
-					 &plaintext, &signature);
+	                                 &plaintext, &signature);
 	if (ret < 0)
 		fail("gnutls_pubkey_verify_data2 failed\n");
 	FIPS_POP_CONTEXT(NOT_APPROVED);
@@ -192,21 +221,23 @@ void sign_verify_unsuccessfully(gnutls_privkey_t privkey,
 	gnutls_fips140_context_deinit(fips_context);
 }
 
-void nosign_verify(gnutls_privkey_t privkey, gnutls_pubkey_t pubkey)
-{
+
+void nosign_verify(gnutls_privkey_t privkey, gnutls_pubkey_t pubkey) {
 	int ret;
 	gnutls_fips140_context_t fips_context;
+	gnutls_fips140_operation_state_t fips_state;
 
 	gnutls_datum_t signature;
 	gnutls_datum_t plaintext = {
-		.data = (unsigned char *const)"Hello world!", .size = 12
+		.data = (unsigned char* const) "Hello world!",
+		.size = 12
 	};
 	assert(gnutls_fips140_context_init(&fips_context) == 0);
 
 	/* 1024, 1280, 1536, 1792 key RSA sign: not approved */
 	FIPS_PUSH_CONTEXT();
 	ret = gnutls_privkey_sign_data(privkey, GNUTLS_DIG_SHA256, 0,
-				       &plaintext, &signature);
+	                               &plaintext, &signature);
 	if (ret < 0)
 		fail("gnutls_privkey_sign_data failed\n");
 	FIPS_POP_CONTEXT(NOT_APPROVED);
@@ -216,7 +247,7 @@ void nosign_verify(gnutls_privkey_t privkey, gnutls_pubkey_t pubkey)
 	assert(gnutls_fips140_mode_enabled() == GNUTLS_FIPS140_LAX);
 
 	ret = gnutls_privkey_sign_data(privkey, GNUTLS_DIG_SHA256, 0,
-				       &plaintext, &signature);
+	                               &plaintext, &signature);
 	if (ret < 0)
 		fail("gnutls_privkey_sign_data failed\n");
 
@@ -226,7 +257,7 @@ void nosign_verify(gnutls_privkey_t privkey, gnutls_pubkey_t pubkey)
 	/* 1024, 1280, 1536, 1792 key RSA verify: approved (exception) */
 	FIPS_PUSH_CONTEXT();
 	ret = gnutls_pubkey_verify_data2(pubkey, GNUTLS_SIGN_RSA_SHA256, 0,
-					 &plaintext, &signature);
+	                                 &plaintext, &signature);
 	if (ret < 0)
 		fail("gnutls_pubkey_verify_data2 failed\n");
 	FIPS_POP_CONTEXT(APPROVED);
@@ -237,6 +268,7 @@ void nosign_verify(gnutls_privkey_t privkey, gnutls_pubkey_t pubkey)
 	gnutls_fips140_context_deinit(fips_context);
 }
 
+
 void doit(void)
 {
 	gnutls_fips140_context_t fips_context;
@@ -245,23 +277,40 @@ void doit(void)
 
 	if (gnutls_fips140_mode_enabled() == 0) {
 		success("We are not in FIPS140 mode\n");
-		exit(77); /* SKIP */
+		exit(77);  /* SKIP */
 	}
 
 	assert(gnutls_fips140_context_init(&fips_context) == 0);
 
+	/* 512-bit RSA: no generate, no sign, no verify */
+	generate_unsuccessfully(&privkey, &pubkey, 512);
+	sign_verify_unsuccessfully(privkey, pubkey);
+	/* 512-bit RSA again (to be safer about going in and out of FIPS) */
+	generate_unsuccessfully(&privkey, &pubkey, 512);
+	sign_verify_unsuccessfully(privkey, pubkey);
+	/* 600-bit RSA: no generate, no sign, no verify */
+	generate_unsuccessfully(&privkey, &pubkey, 600);
+	sign_verify_unsuccessfully(privkey, pubkey);
+
+	/* 768-bit RSA not-an-exception: nogenerate, nosign, verify */
 	generate_unsuccessfully(&privkey, &pubkey, 768);
 	sign_verify_unsuccessfully(privkey, pubkey);
+	/* 1024-bit RSA exception: nogenerate, nosign, verify */
 	generate_unsuccessfully(&privkey, &pubkey, 1024);
-	sign_verify_unsuccessfully(privkey, pubkey);
+	nosign_verify(privkey, pubkey);
+	/* 1280-bit RSA exception: nogenerate, nosign, verify */
 	generate_unsuccessfully(&privkey, &pubkey, 1280);
-	sign_verify_unsuccessfully(privkey, pubkey);
+	nosign_verify(privkey, pubkey);
+	/* 1500-bit RSA not-an-exception: nogenerate, nosign, noverify */
 	generate_unsuccessfully(&privkey, &pubkey, 1500);
 	sign_verify_unsuccessfully(privkey, pubkey);
+	/* 1536-bit RSA exception: nogenerate, nosign, verify */
 	generate_unsuccessfully(&privkey, &pubkey, 1536);
-	sign_verify_unsuccessfully(privkey, pubkey);
+	nosign_verify(privkey, pubkey);
+	/* 1792-bit RSA exception: nogenerate, nosign, verify */
 	generate_unsuccessfully(&privkey, &pubkey, 1792);
-	sign_verify_unsuccessfully(privkey, pubkey);
+	nosign_verify(privkey, pubkey);
+	/* 2000-bit RSA not-an-exception: nogenerate, nosign, noverify */
 	generate_unsuccessfully(&privkey, &pubkey, 2000);
 	sign_verify_unsuccessfully(privkey, pubkey);
 
