@@ -16,13 +16,14 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with GnuTLS.  If not, see <https://www.gnu.org/licenses/>.
+ * along with GnuTLS; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 /* Parts copied from GnuTLS example programs. */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 
 #include <stdio.h>
@@ -54,7 +55,7 @@ void doit(void)
 	if (ret < 0) {
 		fail("Could not initialize: %d\n", __LINE__);
 	}
-
+	
 	/* the rest shouldn't cause a leak */
 	ret = gnutls_global_init();
 	if (ret < 0) {
@@ -77,6 +78,6 @@ void doit(void)
 	if (ret < 0) {
 		fail("Could not initialize: %d\n", __LINE__);
 	}
-
+	
 	gnutls_global_deinit();
 }
