@@ -15,11 +15,12 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with GnuTLS.  If not, see <https://www.gnu.org/licenses/>.
+ * along with GnuTLS; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 
 #include <stdio.h>
@@ -100,14 +101,13 @@ static void client(int fd)
 
 	/* Initialize TLS session
 	 */
-	gnutls_init(&session, GNUTLS_CLIENT | GNUTLS_DATAGRAM);
+	gnutls_init(&session, GNUTLS_CLIENT|GNUTLS_DATAGRAM);
 	gnutls_record_set_timeout(session, 10000);
 
 	/* Use default priorities */
-	ret = gnutls_priority_set_direct(
-		session,
-		"NONE:+VERS-DTLS1.0:+AES-128-CBC:+SHA1:+SIGN-ALL:+COMP-NULL:+ANON-DH:+ANON-ECDH:+CURVE-ALL",
-		&err);
+	ret = gnutls_priority_set_direct(session,
+				   "NONE:+VERS-DTLS1.0:+AES-128-CBC:+SHA1:+SIGN-ALL:+COMP-NULL:+ANON-DH:+ANON-ECDH:+CURVE-ALL",
+				   &err);
 	if (ret < 0) {
 		fail("client: priority set failed (%s): %s\n",
 		     gnutls_strerror(ret), err);
@@ -124,7 +124,8 @@ static void client(int fd)
 	 */
 	do {
 		ret = gnutls_handshake(session);
-	} while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
+	}
+	while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
 
 	if (ret < 0) {
 		fail("client: Handshake failed: %s\n", strerror(ret));
@@ -136,34 +137,30 @@ static void client(int fd)
 
 	if (debug)
 		success("client: TLS version is: %s\n",
-			gnutls_protocol_get_name(
-				gnutls_protocol_get_version(session)));
+			gnutls_protocol_get_name
+			(gnutls_protocol_get_version(session)));
 
 	ret = gnutls_cipher_get(session);
 	if (ret != GNUTLS_CIPHER_AES_128_CBC) {
-		fprintf(stderr, "negotiated unexpected cipher: %s\n",
-			gnutls_cipher_get_name(ret));
+		fprintf(stderr, "negotiated unexpected cipher: %s\n", gnutls_cipher_get_name(ret));
 		exit(1);
 	}
 
 	ret = gnutls_mac_get(session);
 	if (ret != GNUTLS_MAC_SHA1) {
-		fprintf(stderr, "negotiated unexpected mac: %s\n",
-			gnutls_mac_get_name(ret));
+		fprintf(stderr, "negotiated unexpected mac: %s\n", gnutls_mac_get_name(ret));
 		exit(1);
 	}
 
 	/* save state */
-	ret = gnutls_record_get_state(session, 0, NULL, NULL, NULL,
-				      wseq_number);
+	ret = gnutls_record_get_state(session, 0, NULL, NULL, NULL, wseq_number);
 	if (ret < 0) {
 		fprintf(stderr, "error in %d\n", __LINE__);
 		gnutls_perror(ret);
 		exit(1);
 	}
 
-	ret = gnutls_record_get_state(session, 1, NULL, NULL, NULL,
-				      rseq_number);
+	ret = gnutls_record_get_state(session, 1, NULL, NULL, NULL, rseq_number);
 	if (ret < 0) {
 		fprintf(stderr, "error in %d\n", __LINE__);
 		gnutls_perror(ret);
@@ -171,25 +168,19 @@ static void client(int fd)
 	}
 
 	/* skip past the sliding window */
-	for (i = 0; i < 96; i++) {
-		do {
-			ret = gnutls_record_send(session, "hello", 5);
-		} while (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED);
+	for (i=0;i<96;i++) {
+		ret = gnutls_record_send(session, "hello", 5);
 		if (ret < 0) {
 			fail("gnutls_record_send: %s\n", gnutls_strerror(ret));
 		}
 
-		do {
-			ret = gnutls_record_recv(session, buf, sizeof(buf));
-		} while (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED);
+		ret = gnutls_record_recv(session, buf, sizeof(buf));
 		if (ret < 0) {
 			fail("gnutls_record_recv: %s\n", gnutls_strerror(ret));
 		}
 	}
 
-	do {
-		ret = gnutls_record_send(session, "reset", 5);
-	} while (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED);
+	ret = gnutls_record_send(session, "reset", 5);
 	if (ret < 0) {
 		fail("gnutls_record_send(reset): %s\n", gnutls_strerror(ret));
 	}
@@ -208,16 +199,12 @@ static void client(int fd)
 		exit(1);
 	}
 
-	do {
-		ret = gnutls_record_send(session, "ping", 4);
-	} while (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED);
+	ret = gnutls_record_send(session, "ping", 4);
 	if (ret < 0) {
 		fail("gnutls_record_send(ping): %s\n", gnutls_strerror(ret));
 	}
 
-	do {
-		ret = gnutls_record_recv(session, buf, sizeof(buf));
-	} while (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED);
+	ret = gnutls_record_recv(session, buf, sizeof(buf));
 	if (ret < 0) {
 		fail("gnutls_record_recv(pong): %s\n", gnutls_strerror(ret));
 	}
@@ -246,7 +233,8 @@ static void server(int fd)
 	unsigned char rseq_number[8];
 	unsigned char wseq_number[8];
 	char buf[128];
-	const gnutls_datum_t p3 = { (unsigned char *)pkcs3, strlen(pkcs3) };
+	const gnutls_datum_t p3 =
+	    { (unsigned char *) pkcs3, strlen(pkcs3) };
 
 	/* this must be called once in the program
 	 */
@@ -262,14 +250,14 @@ static void server(int fd)
 	gnutls_dh_params_import_pkcs3(dh_params, &p3, GNUTLS_X509_FMT_PEM);
 	gnutls_anon_set_server_dh_params(anoncred, dh_params);
 
-	gnutls_init(&session, GNUTLS_SERVER | GNUTLS_DATAGRAM);
+	gnutls_init(&session, GNUTLS_SERVER|GNUTLS_DATAGRAM);
 	gnutls_record_set_timeout(session, 10000);
 
 	/* avoid calling all the priority functions, since the defaults
 	 * are adequate.
 	 */
-	ret = gnutls_priority_set_direct(
-		session, "NORMAL:+VERS-DTLS1.0:+ANON-DH:+ANON-ECDH", NULL);
+	ret = gnutls_priority_set_direct(session,
+				   "NORMAL:+VERS-DTLS1.0:+ANON-DH:+ANON-ECDH", NULL);
 	if (ret < 0) {
 		fail("server: priority set failed (%s)\n\n",
 		     gnutls_strerror(ret));
@@ -282,7 +270,8 @@ static void server(int fd)
 
 	do {
 		ret = gnutls_handshake(session);
-	} while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
+	}
+	while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
 	if (ret < 0) {
 		close(fd);
 		gnutls_deinit(session);
@@ -295,26 +284,22 @@ static void server(int fd)
 
 	if (debug)
 		success("server: TLS version is: %s\n",
-			gnutls_protocol_get_name(
-				gnutls_protocol_get_version(session)));
+			gnutls_protocol_get_name
+			(gnutls_protocol_get_version(session)));
 
 	/* save state */
-	ret = gnutls_record_get_state(session, 0, NULL, NULL, NULL,
-				      wseq_number);
+	ret = gnutls_record_get_state(session, 0, NULL, NULL, NULL, wseq_number);
 	if (ret < 0) {
 		fail("error in %d\n", __LINE__);
 	}
 
-	ret = gnutls_record_get_state(session, 1, NULL, NULL, NULL,
-				      rseq_number);
+	ret = gnutls_record_get_state(session, 1, NULL, NULL, NULL, rseq_number);
 	if (ret < 0) {
 		fail("error in %d\n", __LINE__);
 	}
 
 	do {
-		do {
-			ret = gnutls_record_recv(session, buf, sizeof(buf));
-		} while (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED);
+		ret = gnutls_record_recv(session, buf, sizeof(buf));
 
 		if (ret > 0) {
 			if (ret == 5 && memcmp(buf, "reset", 5) == 0) {
@@ -323,12 +308,9 @@ static void server(int fd)
 				break;
 			}
 
-			do {
-				ret = gnutls_record_send(session, buf, ret);
-			} while (ret == GNUTLS_E_AGAIN ||
-				 ret == GNUTLS_E_INTERRUPTED);
+			ret = gnutls_record_send(session, buf, ret);
 		}
-	} while (ret > 0);
+	} while(ret > 0);
 
 	if (ret < 0) {
 		fail("error: %s\n", gnutls_strerror(ret));
@@ -344,14 +326,10 @@ static void server(int fd)
 		fail("error in %d\n", __LINE__);
 	}
 
-	do {
-		ret = gnutls_record_recv(session, buf, sizeof(buf));
-	} while (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED);
+	ret = gnutls_record_recv(session, buf, sizeof(buf));
 
 	if (ret == 4 && memcmp(buf, "ping", 4) == 0) {
-		do {
-			ret = gnutls_record_send(session, "pong", 4);
-		} while (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED);
+		ret = gnutls_record_send(session, "pong", 4);
 	} else if (ret > 0) {
 		fail("did not receive ping; received: %.*s\n", ret, buf);
 	} else if (ret < 0) {
@@ -413,4 +391,4 @@ void doit(void)
 	start();
 }
 
-#endif /* _WIN32 */
+#endif				/* _WIN32 */
