@@ -43,6 +43,7 @@ int main()
 #include <sys/wait.h>
 #include <arpa/inet.h>
 #include <unistd.h>
+#include <signal.h>
 #include <gnutls/gnutls.h>
 #include <gnutls/dtls.h>
 
@@ -131,7 +132,7 @@ static void client(int fd, int server_init)
 		do {
 			ret =
 			    gnutls_record_recv(session, buffer,
-					       sizeof(buffer));
+						sizeof(buffer));
 
 			if (ret == GNUTLS_E_HEARTBEAT_PING_RECEIVED) {
 				if (debug)
@@ -141,16 +142,16 @@ static void client(int fd, int server_init)
 				if (ret2 < 0) {
 					fail("pong: %s\n",
 					     gnutls_strerror(ret));
-					terminate();
+					exit(1);
 				}
 			}
 		}
 		while (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED
-		       || ret == GNUTLS_E_HEARTBEAT_PING_RECEIVED);
+			|| ret == GNUTLS_E_HEARTBEAT_PING_RECEIVED);
 
 		if (ret < 0) {
 			fail("recv: %s\n", gnutls_strerror(ret));
-			terminate();
+			exit(1);
 		}
 	} else {
 		do {
@@ -162,11 +163,11 @@ static void client(int fd, int server_init)
 				success("Ping sent.\n");
 		}
 		while (ret == GNUTLS_E_AGAIN
-		       || ret == GNUTLS_E_INTERRUPTED);
+			|| ret == GNUTLS_E_INTERRUPTED);
 
 		if (ret < 0) {
 			fail("ping: %s\n", gnutls_strerror(ret));
-			terminate();
+			exit(1);
 		}
 	}
 
@@ -204,7 +205,7 @@ static gnutls_session_t initialize_tls_session(void)
 static void terminate(void)
 {
 	int status;
-
+	assert(child);
 	kill(child, SIGTERM);
 	wait(&status);
 	exit(1);
@@ -258,7 +259,7 @@ static void server(int fd, int server_init)
 		do {
 			ret =
 			    gnutls_record_recv(session, buffer,
-					       sizeof(buffer));
+						sizeof(buffer));
 
 			if (ret == GNUTLS_E_HEARTBEAT_PING_RECEIVED) {
 				if (debug)
@@ -273,7 +274,7 @@ static void server(int fd, int server_init)
 			}
 		}
 		while (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED
-		       || ret == GNUTLS_E_HEARTBEAT_PING_RECEIVED);
+			|| ret == GNUTLS_E_HEARTBEAT_PING_RECEIVED);
 	} else {
 		do {
 			ret =
@@ -284,7 +285,7 @@ static void server(int fd, int server_init)
 				success("Ping sent.\n");
 		}
 		while (ret == GNUTLS_E_AGAIN
-		       || ret == GNUTLS_E_INTERRUPTED);
+			|| ret == GNUTLS_E_INTERRUPTED);
 
 		if (ret < 0) {
 			fail("ping: %s\n", gnutls_strerror(ret));
@@ -312,6 +313,8 @@ static void start(int server_initiated)
 	int fd[2];
 	int ret;
 
+	signal(SIGPIPE, SIG_IGN);
+
 	ret = socketpair(AF_UNIX, SOCK_STREAM, 0, fd);
 	if (ret < 0) {
 		perror("socketpair");
@@ -331,9 +334,7 @@ static void start(int server_initiated)
 
 		server(fd[0], server_initiated);
 		wait(&status);
-		if (WEXITSTATUS(status) != 0)
-			fail("Child died with status %d\n",
-			     WEXITSTATUS(status));
+		check_wait_status(status);
 	} else {
 		close(fd[0]);
 		client(fd[1], server_initiated);
