@@ -111,6 +111,7 @@ static ssize_t odd_push(gnutls_transport_ptr_t tr, const void *data, size_t len)
 	}
 
 	stored_messages[current] = malloc(len);
+	assert(stored_messages[current]);
 	memcpy(stored_messages[current], data, len);
 	stored_sizes[current] = len;
 
@@ -247,12 +248,12 @@ static void client(int fd)
 
 			if (recv_msg_seq[current] == -1) {
 				fail("received message sequence differs\n");
-				terminate();
+				exit(1);
 			}
 			if (((uint32_t)recv_msg_seq[current]) != useq) {
 				fail("received message sequence differs (current: %u, got: %u, expected: %u)\n",
 				     (unsigned)current, (unsigned)useq, (unsigned)recv_msg_seq[current]);
-				terminate();
+				exit(1);
 			}
 
 			current++;
@@ -275,7 +276,7 @@ static void client(int fd)
 static void terminate(void)
 {
 	int status;
-
+	assert(child);
 	kill(child, SIGTERM);
 	wait(&status);
 	exit(1);
