@@ -83,6 +83,9 @@ void doit(void)
 	gnutls_pubkey_t pubkey4;
 	unsigned i;
 
+	if (gnutls_fips140_mode_enabled())
+		exit(77);
+
 	bin = softhsm_bin();
 
 	lib = softhsm_lib();
