@@ -30,6 +30,7 @@
 #include <signal.h>
 #include <string.h>
 #include <stdarg.h>
+#include <unistd.h>
 #include <gnutls/gnutls.h>
 #include <gnutls/pkcs11.h>
 
@@ -74,11 +75,11 @@ extern const char *pkcs3_3072;
     _fail("%s:%d: "format, __func__, __LINE__, ##__VA_ARGS__)
 
 extern void _fail(const char *format, ...)
-    __attribute__ ((format(printf, 1, 2)));
+	__attribute__ ((format(printf, 1, 2))) __attribute__((__noreturn__));
 extern void fail_ignore(const char *format, ...)
-    __attribute__ ((format(printf, 1, 2)));
+	__attribute__ ((format(printf, 1, 2))) __attribute__((__noreturn__));
 extern void success(const char *format, ...)
-    __attribute__ ((format(printf, 1, 2)));
+	__attribute__ ((format(printf, 1, 2)));
 
 /* assumes test_name is defined */
 #define test_fail(fmt, ...) \
@@ -146,6 +147,8 @@ char *get_tmpname(char s[TMPNAME_SIZE]);
 void track_temp_files(void);
 void delete_temp_files(void);
 
+int tcp_connect(const char* addr, unsigned port);
+
 /* This must be implemented elsewhere. */
 extern void doit(void);
 
@@ -160,7 +163,7 @@ inline static void _check_wait_status(int status, unsigned sigonly)
 		} else {
 			if (!sigonly) {
 				if (WEXITSTATUS(status) == 77)
-					exit(77);
+					_exit(77);
 				fail("Child died with status %d\n",
 				     WEXITSTATUS(status));
 			}
